from .common_message_segment import Image as Image
from .common_message_segment import Mention as Mention
from .common_message_segment import Reply as Reply
from .common_message_segment import Text as Text
from .custom_message_segment import Custom as Custom



from nonebot import require

require("nonebot_plugin_htmlrender")
require("nonebot_plugin_templates")
require("nonebot_plugin_web_config")
require("nonebot_plugin_alconna")
require("nonebot_plugin_bind")

from . import nonebot  # noqa: F401, E402
from .chatbots import BingBot, chatbots  # noqa: F401, E402
from .type_store import users  # noqa: F401, E402



def is_valid_name(s):
    count = 0
    for c in s:
        if c.isalnum() or (c in ["_", "-"]):
            count += 1
        else:
            raise Exception("chat_name 会话名称 只能由数字,字母,汉字,'_','-' 组成")
        if count > 10:
            raise Exception("chat_name 会话名称 中的字符数量不能多于10个")
    return True



from functools import partial
from io import BytesIO
from pathlib import Path
from typing import Any, Dict, Optional

from nonebot.adapters import Bot as BaseBot
from nonebot.adapters import Event

from ..types import Text, Image, Reply, Mention
from ..utils import (
    MessageFactory,
    PlatformTarget,
    SupportedAdapters,
    SupportedPlatform,
    register_get_bot_id,
    TargetDiscordChannel,
    MessageSegmentFactory,
    register_sender,
    register_ms_adapter,
    register_convert_to_arg,
    assamble_message_factory,
    register_target_extractor,
    Receipt, get_bot_id,
)

try:
    import aiohttp
    from nonebot.adapters.discord import Bot
    from nonebot.adapters.discord.bot import parse_message
    from nonebot.adapters.discord.message import Message, MessageSegment
    from nonebot.adapters.discord.event import (
        MessageEvent,
        MessageCreateEvent,
        ChannelPinsUpdateEvent,
    )

    adapter = SupportedAdapters.discord
    register_discord = partial(register_ms_adapter, adapter)

    MessageFactory.register_adapter_message(SupportedAdapters.discord, Message)


    @register_discord(Text)
    def _text(t: Text) -> MessageSegment:
        return MessageSegment.text(t.data["text"])


    @register_discord(Image)
    async def _image(i: Image, bot: BaseBot) -> MessageSegment:
        if not isinstance(bot, Bot):
            raise TypeError(f"Unsupported type of bot: {type(bot)}")
        image = i.data["image"]
        img_bytes = b""
        if isinstance(image, (str, Path)):
            path = Path(image)
            if i.data["name"] == "image":
                if path.suffix not in [".jpg", ".jpeg", ".png", ".gif"]:
                    i.data["name"] = path.with_suffix(".png").name
                else:
                    i.data["name"] = path.name
            if path.exists():
                try:
                    with path.open("rb") as f:
                        img_bytes = f.read()
                except Exception as error:
                    raise Exception(f"Error reading local image: {str(error)}")
            else:
                async with aiohttp.ClientSession() as session:
                    async with session.get(str(image)) as response:
                        if response.status == 200:
                            img_bytes = await response.read()
                        else:
                            raise Exception(
                                f"Error downloading image, status code: {response.status}"  # noqa: E501
                            )
        elif isinstance(image, bytes):
            img_bytes = image
        elif isinstance(image, BytesIO):
            img_bytes = image.read()
        else:
            raise TypeError("Invalid image type")

        return MessageSegment.attachment(
            content=img_bytes,
            file=i.data["name"] if i.data["name"] != "image" else "image.png",
        )


    @register_discord(Mention)
    async def _mention(m: Mention) -> MessageSegment:
        return MessageSegment.mention_user(user_id=int(m.data["user_id"]))


    @register_discord(Reply)
    async def _reply(r: Reply) -> MessageSegment:
        return MessageSegment.reference(reference=int(r.data["message_id"]))


    @register_target_extractor(ChannelPinsUpdateEvent)
    @register_target_extractor(MessageCreateEvent)
    @register_target_extractor(MessageEvent)
    def _extract_msg_event(event: Event) -> TargetDiscordChannel:
        assert isinstance(event, MessageEvent)
        return TargetDiscordChannel(channel_id=event.channel_id)


    @register_convert_to_arg(adapter, SupportedPlatform.discord_channel)
    def _gen_channel(target: PlatformTarget) -> Dict[str, Any]:
        assert isinstance(target, TargetDiscordChannel)
        return {
            "channel_id": target.channel_id,
        }


    class DiscordReceipt(Receipt):
        sent_msg: dict
        message_id: int
        channel_id: int
        reply_to_msg_id: Optional[int]
        mention_user_id: Optional[str]
        adapter_name = adapter

        async def revoke(self):
            return await self._get_bot().call_api(
                "delete_message",
                channel_id=self.channel_id,
                message_id=self.message_id,
            )

        async def edit(self, msg: MessageFactory[MessageSegmentFactory], *, at_sender=False, reply=False):
            bot = self._get_bot()
            if at_sender and self.mention_user_id:
                msg = Mention(self.mention_user_id) + msg
            if reply and self.reply_to_msg_id:
                msg = Reply(self.reply_to_msg_id) + msg
            message_to_send = Message()
            for message_segment_factory in msg:
                message_segment = await message_segment_factory.build(bot)
                message_to_send += message_segment
            message_data = parse_message(message_to_send)
            del message_data["sticker_ids"]
            del message_data["message_reference"]
            return await bot.edit_message(channel_id=self.channel_id,
                                          message_id=self.message_id, **message_data)

        @property
        def raw(self) -> Any:
            return self.sent_msg


    @register_sender(SupportedAdapters.discord)
    async def send(
        bot: Bot,
        msg: MessageFactory[MessageSegmentFactory],
        target,
        event,
        at_sender: bool,
        reply: bool,
    ) -> DiscordReceipt:
        assert isinstance(bot, Bot)
        assert isinstance(target, TargetDiscordChannel)
        params = {"channel_id": target.channel_id}
        if at_sender:
            params['mention_user_id'] = event.get_user_id()
        if reply:
            params['reply_to_msg_id'] = event.message_id

        if event:
            assert isinstance(event, MessageEvent)
            full_msg = assamble_message_factory(
                msg,
                Mention(event.get_user_id()),
                Reply(event.message_id),
                at_sender,
                reply,
            )
        else:
            full_msg = msg
        message_to_send = Message()
        for message_segment_factory in full_msg:
            message_segment = await message_segment_factory.build(bot)
            message_to_send += message_segment

        sent_msg = await bot.send_to(message=message_to_send, **target.arg_dict(bot))
        return DiscordReceipt(bot_id=get_bot_id(bot), sent_msg=sent_msg.dict(), message_id=sent_msg.id, **params)


    @register_get_bot_id(adapter)
    def _get_id(bot: BaseBot):
        assert isinstance(bot, Bot)
        return bot.self_id
except ImportError:
    pass
except Exception as e:
    raise e



import asyncio
from typing import AsyncGenerator

from async_claude_client import Slack_Claude_Client, Text, ChatUpdate

from . import register_chatbot
from .BaseChatBot import BaseChatBot, Permission
from ..type_store.web_config import slack_claude_config

config = slack_claude_config()
client: Slack_Claude_Client = None
slack_user_token = config.slack_user_token
channel_id = config.channel_id
claude_id = config.claude_id
pre_msg = config.pre_msg
locks = {}

@register_chatbot
class Slack_ClaudeBot(BaseChatBot):
    model = "Slack_Claude"
    permission = Permission.common
    chat: dict = {}

    @classmethod
    @property
    def desc(self) -> str:
        return "Slack.com 的 Claude 模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = slack_claude_config()
        return bool(config.channel_id and config.claude_id and config.slack_user_token)

    async def ensure(self):
        global client, slack_user_token, channel_id, claude_id, pre_msg, locks
        config = slack_claude_config()
        if self.__hash__() not in locks.keys():
            locks[self.__hash__()] = asyncio.Lock()
        if (
            (not client)
            or config.slack_user_token != slack_user_token
            or config.channel_id != channel_id
            or config.claude_id != claude_id
            or config.pre_msg != pre_msg
        ):
            slack_user_token = config.slack_user_token
            channel_id = config.channel_id
            claude_id = config.claude_id
            pre_msg = config.pre_msg
            client = Slack_Claude_Client(
                slack_user_token, claude_id, channel_id, pre_msg
            )

    async def refresh(self):
        await self.ensure()
        async with locks[self.__hash__()]:
            self.chat = client.create_new_chat()
            self.save()
        if self.prompt:
            async for text in self.ask(self.prompt + "\n第一次回复只需要回复'我明白了',只需要回复'我明白了'"):
                pass

    async def ask(self, question: str) -> AsyncGenerator:
        question = self.process_question(question)
        await self.ensure()
        if not self.chat:
            await self.refresh()
        async with locks[self.__hash__()]:
            async for data in client.ask_stream_raw(question, self.chat):
                if isinstance(data, Text):
                    yield data.content
                elif isinstance(data, ChatUpdate):
                    self.chat = data.content
                    self.save()



import asyncio
from enum import Enum
from typing import Optional, AsyncGenerator

from ..nonebot_plugin_saa import MessageFactory, Text, Image
from pydantic import BaseModel

from ..type_store.web_config import common_config
from ..utils.pastebin import get_url
from ..utils.render import md_to_pic
from ..type_store.msgs_link import msg_links


class Permission(str, Enum):
    black = "black"
    common = "common"
    paid = "paid"


class RateLimit(BaseModel):
    type: str
    limit: int


class BaseChatBot(BaseModel):
    name: str
    owner_id: str
    model: str
    permission: Permission
    prompt_name: Optional[str]
    prompt: Optional[str]
    command_name: Optional[str]
    command: Optional[str]
    stream: Optional[bool] = False
    url: Optional[bool] = False
    pic: Optional[bool] = False
    auto_pic: Optional[bool] = True
    num_limit: Optional[int] = 500

    def __hash__(self):
        return hash(str(self.owner_id) + self.name)

    @classmethod
    @property
    def desc(self) -> str:
        raise NotImplementedError

    @classmethod
    @property
    def able(cls) -> bool:
        raise NotImplementedError

    def process_question(self, question):
        if self.command:
            return self.command + "\n" + question
        else:
            return question

    async def ask_plain(self, *args, **kwargs):
        full_answer = ""
        async for text in self.ask(*args, **kwargs):
            full_answer += text

        message = []

        async def md_to_pic_task(s, width=common_config().pic_width):
            if (self.auto_pic and len(full_answer) > self.num_limit) or self.pic:
                pic = await md_to_pic(s, width)
                return Image(pic)
            else:
                return Text(s)

        async def get_url_task(s):
            if (
                (self.auto_pic and len(full_answer) > self.num_limit) or self.pic
            ) and self.url:
                url = await get_url(s)
                return Text(url)

        tasks = []
        tasks.append(asyncio.create_task(md_to_pic_task(full_answer)))
        tasks.append(asyncio.create_task(get_url_task(full_answer)))

        results = await asyncio.gather(*tasks)

        for result in results:
            if result:
                message.append(result)

        receipt = await MessageFactory(message).send(at_sender=True)
        msg_links.add_msglink(self.owner_id, receipt.message_id, self)

    async def ask_stream(self, *args, **kwargs):
        full_answer = ""
        answer = ""
        receipt = None
        async for text in self.ask(*args, **kwargs):
            full_answer += text
            answer += text
            times = 0
            if "\n" in answer:
                answer_slice = answer.rsplit("\n", 1)
                if len(answer_slice[0].replace(" ", "").replace("\n", "")) > 80:
                    if receipt and receipt.edit_able:
                        await receipt.edit(Text(full_answer), at_sender=True)
                    else:
                        receipt = await MessageFactory(Text(answer_slice[0])).send(
                            at_sender=True
                        )
                        msg_links.add_msglink(self.owner_id, receipt.message_id, self)
                    times += 1
                    answer = answer_slice[1]
        if answer:
            times += 1
            if receipt and receipt.edit_able:
                await receipt.edit(Text(full_answer), at_sender=True)
            else:
                receipt = await MessageFactory(Text(answer)).send(at_sender=True)
                msg_links.add_msglink(self.owner_id, receipt.message_id, self)

        if times > 1 and self.url:
            url = await get_url(full_answer)
            if receipt and receipt.edit_able:
                await receipt.edit(Text(full_answer), at_sender=True)
            else:
                receipt = await MessageFactory(Text(url)).send(at_sender=True)
                msg_links.add_msglink(self.owner_id, receipt.message_id, self)

    async def ask(self, *args, **kwargs) -> AsyncGenerator:
        raise NotImplementedError

    async def refresh(self, *args, **kwargs):
        raise NotImplementedError

    def save(self):
        from ..type_store.user_chat import users

        users[self.owner_id].save()



from nonebot import on_message, Bot
from nonebot.params import Event

from ..nonebot_plugin_saa import MessageFactory, Text
from ..type_store import common_config
from ..type_store.msgs_link import msg_links
from ..utils.extractor import extract_image, QCU

chat_consumer = on_message(block=False)


@chat_consumer.handle()
async def __(q_c: QCU, event: Event, bot: Bot):
    question, chat, user = q_c
    toev_receipt = None
    receipt = None
    if not chat.able:
        await MessageFactory(Text("该会话的模型引擎已被禁用,目前无法使用.")).finish(at_sender=True)

    if not question.replace(" ", "").replace("\n", ""):
        receipt = await MessageFactory(Text("你没有输入问题,请重新询问并在命令后或回复中加上问题.")).finish(
            at_sender=True
        )

    if len(question) < 7 and question.replace(" ", "").replace("\n", "").replace(
        "\r", ""
    ) in ["刷新对话", "清除历史", "遗忘"]:
        if common_config().wait_msg_able:
            toev_receipt = await MessageFactory(Text("正在刷新,请稍等~~")).send(at_sender=True)
        try:
            await chat.refresh()
            receipt = await MessageFactory("刷新成功").send(at_sender=True)
            msg_links.add_msglink(user.user_id, receipt.message_id, chat)
        except Exception as e:
            if receipt:
                await receipt.revoke()
            receipt = await MessageFactory(Text(f"在刷新过程中出错: {str(e)[:200]}")).send(
                at_sender=True
            )
            msg_links.add_msglink(user.user_id, receipt.message_id, chat)
            raise e
    else:
        try:
            if common_config().wait_msg_able:
                toev_receipt = await MessageFactory(Text("正在思考,请稍等~~")).send(
                    at_sender=True
                )
            if chat.model == "Bing":
                image = await extract_image(event, bot)
                if chat.stream:
                    await chat.ask_stream(question, image=image)
                else:
                    await chat.ask_plain(question, image)
            else:
                if chat.stream:
                    await chat.ask_stream(question)
                else:
                    await chat.ask_plain(question)
        except Exception as e:
            if receipt:
                await receipt.revoke()
            receipt = await MessageFactory(Text(f"在聊天过程中出错: {str(e)[:200]}")).send(
                at_sender=True
            )
            msg_links.add_msglink(user.user_id, receipt.message_id, chat)
            raise e
    if toev_receipt:
        await toev_receipt.revoke()



import asyncio
from asyncio import Lock
from typing import AsyncGenerator

from async_poe_client import Poe_Client, ChatCodeUpdate, Text, SuggestRely

from . import register_chatbot
from .BaseChatBot import BaseChatBot, Permission
from ..type_store.web_config import poe_config

config = poe_config()

p_b = config.p_b
form_key = config.formkey
proxy = config.proxy
suggest_able = config.suggest_able
client: Poe_Client = None
url_botname = ["ChatGPT"]
locks = {}


class PoeBot(BaseChatBot):
    url_botname: str
    chat_code: str = None
    suggests: list = None

    async def ensure(self, chat_code: str = ""):
        global client, p_b, form_key, suggest_able, proxy, locks
        if self.__hash__() not in locks.keys():
            locks[self.__hash__()] = Lock()
        config = poe_config()
        suggest_able = config.suggest_able
        if (
            (not client)
            or (p_b != config.p_b)
            or (form_key != config.formkey)
            or (proxy != config.proxy)
        ):
            p_b = config.p_b
            form_key = config.formkey
            proxy = config.proxy
            client = await Poe_Client(p_b=p_b, formkey=form_key, proxy=proxy).create()

    async def refresh(self):
        await self.ensure(self.chat_code)
        if self.chat_code:
            async with locks[self.__hash__()]:
                await client.send_chat_break(
                    url_botname=self.url_botname, chat_code=self.chat_code
                )
        if self.prompt:
            async for data in client.ask_stream_raw(
                url_botname=self.url_botname,
                question=self.prompt + "\n首次回复,只需要回复'我知道了',只需要回复'我知道了'",
                suggest_able=False,
                chat_code=self.chat_code,
            ):
                if isinstance(data, ChatCodeUpdate):
                    self.chat_code = data.content
                    self.save()

    async def ask(self, question: str) -> AsyncGenerator:
        await self.ensure(self.chat_code)
        if (not self.chat_code) and self.prompt:
            await self.refresh()
        if question.isdigit():
            if int(question) <= len(self.suggests):
                question = self.suggests[int(question) - 1]
                
        question = self.process_question(question)
        
        suggests = []
        if self.chat_code:
            while locks[self.__hash__()].locked():
                await asyncio.sleep(1)
        async for data in client.ask_stream_raw(
            url_botname=self.url_botname,
            chat_code=self.chat_code,
            question=question,
            suggest_able=suggest_able,
        ):
            if isinstance(data, Text):
                yield data.content
            elif suggest_able:
                if isinstance(data, SuggestRely):
                    suggests.append(data.content)
            if isinstance(data, ChatCodeUpdate):
                self.chat_code = data.content
                self.save()
        suggest_str = ""
        if suggests:
            suggest_str += "\n\nSuggest Replys:\n"
            index = 1
            for suggest in suggests:
                suggest_str += f"{index}: {suggest}\n"
                index += 1
            yield suggest_str
        self.suggests = suggests


@register_chatbot
class PoeChatGPTBot(PoeBot):
    model = "PoeChatGPT"
    url_botname = "ChatGPT"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的ChatGPT模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey)


@register_chatbot
class PoeClaudeBot(PoeBot):
    model = "PoeClaude"
    url_botname = "Claude-instant"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的Claude-instant模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey)


@register_chatbot
class PoeAssistantBot(PoeBot):
    model = "PoeAssistant"
    url_botname = "Assistant"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于ChatGPT的Assistant模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Assistant)


@register_chatbot
class PoeLlama_2_7bBot(PoeBot):
    model = "PoeLlama_2_7b"
    url_botname = "Llama-2-7b"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Llama-2-7b模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Llama_2_7b)


@register_chatbot
class PoeLlama_2_13bBot(PoeBot):
    model = "PoeLlama_2_13b"
    url_botname = "Llama-2-13b"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Llama-2-13b模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Llama_2_13b)


@register_chatbot
class PoeLlama_2_70bBot(PoeBot):
    model = "PoeLlama_2_70b"
    url_botname = "Llama-2-70b"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Llama-2-70b模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Llama_2_70b)


@register_chatbot
class PoeCode_Llama_7bBot(PoeBot):
    model = "PoeCode_Llama_7b"
    url_botname = "Code-Llama-7b"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Code-Llama-7b模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Code_Llama_7b)


@register_chatbot
class PoeCode_Llama_13bBot(PoeBot):
    model = "PoeCode_Llama_13b"
    url_botname = "Code-Llama-13b"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Code-Llama-13b模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Code_Llama_13b)


@register_chatbot
class PoeCode_Llama_34bBot(PoeBot):
    model = "PoeCode_Llama_34b"
    url_botname = "Code-Llama-34b"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Code-Llama-34b模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Code_Llama_34b)


@register_chatbot
class PoeGoogle_PaLMBot(PoeBot):
    model = "PoeGoogle_PaLM"
    url_botname = "Google-PaLM"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Google-PaLM模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Google_PaLM)


@register_chatbot
class PoeChatGPT_16kBot(PoeBot):
    model = "PoeChatGPT_16k"
    url_botname = "ChatGPT-16k"
    permission = Permission.paid

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Google-PaLM模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_ChatGPT_16k)


@register_chatbot
class PoeGPT_4Bot(PoeBot):
    model = "PoeGPT_4"
    url_botname = "GPT-4"
    permission = Permission.paid

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于GPT-4模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_GPT_4)


@register_chatbot
class PoeGPT_4_32kBot(PoeBot):
    model = "PoeGPT_4_32k"
    url_botname = "GPT-4-32k"
    permission = Permission.paid

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于GPT-4-32k模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_GPT_4_32k)


@register_chatbot
class PoeClaude_instant_100kBot(PoeBot):
    model = "PoeClaude_instant_100k"
    url_botname = "Claude-instant-100k"
    permission = Permission.paid

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Claude-instant-100k模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Claude_instant_100k)


@register_chatbot
class PoeClaude_2_100kBot(PoeBot):
    model = "PoeClaude_2_100k"
    url_botname = "Claude-2-100k"
    permission = Permission.paid

    @classmethod
    @property
    def desc(self) -> str:
        return "Poe的免费的无限制的基于Claude_2_100k模型"

    @classmethod
    @property
    def able(self) -> bool:
        config = poe_config()
        return bool(config.p_b and config.formkey and config.enable_Claude_2_100k)



from arclet.alconna import Alconna, Args, Option, Arparma  # noqa: F401
from nonebot_plugin_alconna import on_alconna

from ..nonebot_plugin_saa import MessageFactory, Image
from . import (
    chat_consumer,
    chat_manager,
    command_manager,
    prompt_manager,
    model_manager,
)  # noqa: F401
from ..const.menus import menus
from ..utils import Command_Start

shelp = on_alconna(Alconna(f"{Command_Start}shelp"))


@shelp.handle()
async def shelp_():
    image = await menus.get_menu_image("all")
    await MessageFactory(Image(image)).send()



from functools import partial
from io import BytesIO
from pathlib import Path
from typing import cast, Any, Union, Optional

from nonebot.adapters import Bot as BaseBot
from nonebot.adapters import Event

from ..types import Text, Image, Reply, Mention
from ..utils import (
    MessageFactory,
    SupportedAdapters,
    MessageSegmentFactory,
    register_sender,
    register_get_bot_id,
    register_ms_adapter,
    assamble_message_factory,
    register_target_extractor, Receipt, get_bot_id,
)
from ..utils.platform_send_target import TargetFeishuGroup, TargetFeishuPrivate

try:
    import httpx
    from nonebot.adapters.feishu import (
        Bot,
        Message,
        MessageEvent,
        MessageSegment,
        GroupMessageEvent,
        MessageSerializer,
        PrivateMessageEvent,
    )

    adapter = SupportedAdapters.feishu
    register_feishu = partial(register_ms_adapter, adapter)

    MessageFactory.register_adapter_message(SupportedAdapters.feishu, Message)


    @register_feishu(Text)
    def _text(t: Text) -> MessageSegment:
        return MessageSegment.text(t.data["text"])


    @register_feishu(Image)
    async def _image(i: Image, bot: BaseBot) -> MessageSegment:
        if not isinstance(bot, Bot):
            raise TypeError(f"Unsupported type of bot: {type(bot)}")

        image = i.data["image"]
        if isinstance(image, str):
            async with httpx.AsyncClient() as client:
                resp = await client.get(image, timeout=10, follow_redirects=True)
                resp.raise_for_status()
                image = resp.content
        elif isinstance(image, Path):
            image = image.read_bytes()
        elif isinstance(image, BytesIO):
            image = image.getvalue()

        data = {"image_type": "message"}
        files = {"image": ("file", image)}
        params = {"method": "POST", "data": data, "files": files}
        result = await bot.call_api("im/v1/images", **params)
        file_key = result["image_key"]
        return MessageSegment.image(file_key)


    @register_feishu(Mention)
    def _mention(m: Mention) -> MessageSegment:
        return MessageSegment.at(m.data["user_id"])


    @register_feishu(Reply)
    def _reply(r: Reply) -> MessageSegment:
        return MessageSegment("reply", cast(dict, r.data))


    @register_target_extractor(PrivateMessageEvent)
    def _extract_private_msg_event(event: Event) -> TargetFeishuPrivate:
        assert isinstance(event, PrivateMessageEvent)
        return TargetFeishuPrivate(open_id=event.get_user_id())


    @register_target_extractor(GroupMessageEvent)
    def _extract_channel_msg_event(event: Event) -> TargetFeishuGroup:
        assert isinstance(event, GroupMessageEvent)
        return TargetFeishuGroup(chat_id=event.event.message.chat_id)


    class FeishuReceipt(Receipt):
        sent_msg: Optional[dict]
        message_id: Union[str, int]
        mention_user_id: Optional[Union[str, int]]
        adapter_name = adapter

        async def revoke(self):
            return await self._get_bot().call_api(f"im/v1/messages/{self.message_id}", **{
                "method": "DELETE",
            })

        async def edit(self, msg, at_sender=False, reply=False):
            bot = self._get_bot()
            if at_sender:
                msg = msg + Mention(user_id=self.mention_user_id)

            message_to_send = Message()
            for message_segment_factory in msg:
                message_segment = await message_segment_factory.build(bot)
                message_to_send += message_segment

            msg_type, content = MessageSerializer(message_to_send).serialize()

            params = {
                "method": "PUT",
                "body": {
                    "content": content,
                    "msg_type": msg_type,
                },
            }
            await bot.call_api(f"im/v1/messages/{self.message_id}", **params)

        @property
        def raw(self) -> Any:
            return self.sent_msg


    @register_sender(SupportedAdapters.feishu)
    async def send(
        bot,
        msg: MessageFactory[MessageSegmentFactory],
        target,
        event,
        at_sender: bool,
        reply: bool,
    ) -> FeishuReceipt:
        assert isinstance(bot, Bot)
        assert isinstance(target, (TargetFeishuPrivate, TargetFeishuGroup))

        if event:
            assert isinstance(event, MessageEvent)
            full_msg = assamble_message_factory(
                msg,
                (
                    Mention(event.get_user_id())
                    if isinstance(event, GroupMessageEvent)
                    else None
                ),
                Reply(event.event.message.message_id),
                at_sender,
                reply,
            )
        else:
            full_msg = msg

        reply_to_message_id = None
        message_to_send = Message()
        for message_segment_factory in full_msg:
            if isinstance(message_segment_factory, Reply):
                reply_to_message_id = message_segment_factory.data["message_id"]
                continue

            message_segment = await message_segment_factory.build(bot)
            message_to_send += message_segment

        msg_type, content = MessageSerializer(message_to_send).serialize()

        if reply_to_message_id is None:
            if isinstance(target, TargetFeishuGroup):
                receive_id, receive_id_type = target.chat_id, "chat_id"
            else:
                receive_id, receive_id_type = target.open_id, "open_id"
            params = {
                "method": "POST",
                "query": {"receive_id_type": receive_id_type},
                "body": {
                    "receive_id": receive_id,
                    "content": content,
                    "msg_type": msg_type,
                },
            }
            sent_msg = await bot.call_api("im/v1/messages", **params)

        else:
            params = {
                "method": "POST",
                "body": {"content": content, "msg_type": msg_type},
            }
            sent_msg = await bot.call_api(f"im/v1/messages/{reply_to_message_id}/reply", **params)  # noqa: E501
        return FeishuReceipt(bot_id=get_bot_id(bot), sent_msg=sent_msg, message_id=sent_msg["message_id"],
                             mention_user_id=event.get_user_id())


    @register_get_bot_id(adapter)
    def _get_id(bot: BaseBot):
        assert isinstance(bot, Bot)
        return bot.self_id


    @register_get_bot_id(adapter)
    def _get_id(bot: BaseBot):
        assert isinstance(bot, Bot)
        return bot.self_id

except ImportError:
    pass
except Exception as e:
    raise e



from arclet.alconna import Args, Option, Alconna, Subcommand
from nonebot.exception import FinishedException
from nonebot_plugin_alconna import on_alconna, AlcResult
from ..nonebot_plugin_saa import MessageFactory, Text, Image
from nonebot_plugin_spark_gpt.const import menus

from ..chatbots import chatbots
from ..type_store import common_config
from ..type_store import prompt_config, command_config, public_user
from ..utils import Command_Start
from ..utils import is_valid_name
from ..utils.extractor import A_User
chat = on_alconna(
    Alconna(
        f"{Command_Start}chat",
        Subcommand("help"),
        Subcommand(
            "list",
            Option("-help"),
        ),
        Subcommand(
            "add",
            Option("-n", Args["chat_name", str]),
            Option("-m", Args["model", str]),
            Option("-p", Args["prompt", str]),
            Option("-c", Args["command", str]),
            Option("-auto_pic", Args["auto_pic", bool]),
            Option("-num_limit", Args["num_limit", int]),
            Option("-pic", Args["pic", bool]),
            Option("-url", Args["url", bool]),
            Option("-stream", Args["stream", bool]),
            Option("-public"),
            Option("-help"),
        ),
        Subcommand(
            "edit",
            Option("-on", Args["old_name", str]),
            Option("-nn", Args["new_name", str]),
            Option("-m", Args["model", str]),
            Option("-p", Args["prompt", str]),
            Option("-c", Args["command", str]),
            Option("-auto_pic", Args["auto_pic", bool]),
            Option("-num_limit", Args["num_limit", int]),
            Option("-pic", Args["pic", bool]),
            Option("-url", Args["url", bool]),
            Option("-stream", Args["stream", bool]),
            Option("-public"),
            Option("-help"),
        ),
        Subcommand(
            "del",
            Option("-n", Args["chat_name", str]),
            Option("-public"),
            Option("-help"),
        ),
    ),
    aliases={"会话"},
)


@chat.assign("$main")
async def chat_():
    await MessageFactory(Text(menus.get_menu_string("chat"))).finish()


@chat.assign("help")
async def chat_help():
    await MessageFactory(Image(await menus.get_menu_image("chat"))).finish()


@chat.assign("list")
async def chat_list(
    user: A_User,
    arp: AlcResult,
):
    if arp.result.find("list.help"):
        await MessageFactory(Text(menus.get_func_string("chat.list"))).finish()
    await MessageFactory(Image(await user.get_bots_img())).finish(reply=True)


@chat.assign("add")
async def chat_add(arp: AlcResult, user: A_User):
    if arp.result.find("add.help"):
        await MessageFactory(Text(menus.get_func_string("chat.add"))).finish()

    create_params = arp.result.all_matched_args
    name = create_params.get("chat_name", None)
    model = create_params.get("model", "")

    prompt = create_params.get("prompt", "")
    command = create_params.get("command", "")
    auto_pic = create_params.get("auto_pic", True)
    num_limit = create_params.get("num_limit", 500)
    pic = create_params.get("pic", False)
    url = create_params.get("url", True)
    stream = create_params.get("stream", True)
    public = arp.result.find("add.public")

    if not (name or model):
        result = "命令有误:\n格式:chat add -n {chat_name} -m {model} (-p {prompt} -c {command} -auto_pic {bool} -num_limit {int} -pic {bool} -url {bool} -stream {bool} -public)\n错误如下:\n "
        if not name:
            result += "参数 name 未填写\n"
        if not model:
            result += "参数 model 未填写\n"
        result += "\n如有疑问,请使用'chat help'查看图片版详细说明"
        await MessageFactory(Text(result)).finish(reply=True)
    else:
        try:
            is_valid_name(name)

            if public:
                if user.user_id in common_config().superusers:
                    user = public_user
                else:
                    await MessageFactory(Text("你无权管理公有对话")).finish(reply=True)
            if prompt.startswith("."):
                prompt_name = prompt[1:]
                prompt = prompt_config()[prompt_name]
            elif prompt:
                prompt_name = "自定义预设"
            else:
                prompt_name = "无"

            if command.startswith("."):
                command_name = command[1:]
                command = command_config()[command_name]
            elif command:
                command_name = "自定义指令"
            else:
                command_name = "无"

            class_ = chatbots.get_model_class(model)

            user.bots[name] = class_(
                name=name,
                prompt_name=prompt_name,
                prompt=prompt,
                command_name=command_name,
                command=command,
                owner_id=user.user_id,
                stream=stream,
                pic=pic,
                url=url,
                auto_pic=auto_pic,
                num_limit=num_limit,
            )
            user.save()

            await MessageFactory(Text(f"成功添加{'公有' if public else '私有'}会话.")).finish(
                reply=True
            )
        except FinishedException:
            ...
        except Exception as e:
            await MessageFactory(Text(f"添加失败: {e}")).finish(reply=True)


@chat.assign("edit")
async def chat_edit(arp: AlcResult, user: A_User):
    if arp.result.find("edit.help"):
        await MessageFactory(Text(menus.get_func_string("chat.edit"))).finish()

    edit_params = arp.result.all_matched_args

    old_name = edit_params.get("old_name", None)
    new_name = edit_params.get("new_name", old_name)
    public = arp.result.find("edit.public")

    if public:
        if user.user_id in common_config().superusers:
            user = public_user
        else:
            await MessageFactory(Text("你无权管理公有对话")).finish(reply=True)

    if old_name not in user.bots.keys():
        await MessageFactory(Text("原会话不存在.")).finish(reply=True)
    else:
        origin_params = user.bots[old_name].dict()

    prompt = edit_params.get("prompt")
    command = edit_params.get("command")
    prompt_name = origin_params.get("prompt_name")
    if prompt and prompt.startswith("."):
        prompt_name = prompt[1:]
        prompt = prompt_config()[prompt_name]
    elif prompt:
        prompt_name = "自定义预设"
    else:
        prompt = origin_params.get("prompt", "")

    command_name = origin_params.get("command_name")
    if command and command.startswith("."):
        command_name = command[1:]
        command = command_config()[command_name]
    elif command:
        command_name = "自定义指令"
    else:
        command = origin_params.get("command", "")

    model = edit_params.get("model", origin_params.get("model"))

    chatbot_params = {
        "name": new_name,
        "owner_id": origin_params.get("owner_id"),
        "prompt": prompt,
        "prompt_name": prompt_name,
        "command": command,
        "command_name": command_name,
        "auto_pic": edit_params.get("auto_pic", origin_params.get("auto_pic", True)),
        "num_limit": edit_params.get("num_limit", origin_params.get("num_limit", 500)),
        "pic": edit_params.get("pic", origin_params.get("pic", False)),
        "url": edit_params.get("url", origin_params.get("url", True)),
        "stream": edit_params.get("stream", origin_params.get("url", False)),
    }

    if not old_name:
        result = "命令有误:\n格式:chat edit -on {old_name} (-nn {new_name} -m {model} -p {prompt} -c {command} -auto_pic {bool} -num_limit {int} -pic {bool} -url {bool} -stream {bool} -public)\n错误如下:\n参数 old_name 未填写\n\n如有疑问,请使用'chat help'查看图片版详细说明"
        await MessageFactory(Text(result)).finish(reply=True)
    else:
        try:
            is_valid_name(new_name)

            if old_name not in user.bots:
                await MessageFactory(Text("原会话不存在")).finish(reply=True)

            class_ = chatbots.get_model_class(model)

            del user.bots[old_name]

            user.bots[new_name] = class_(**chatbot_params)

            user.save()

            await MessageFactory(Text(f"修改{'公有' if public else '私有'}会话成功.")).finish(
                reply=True
            )

        except FinishedException:
            ...
        except Exception as e:
            await MessageFactory(Text(f"修改{'公有' if public else '私有'}会话失败: {e}")).finish(
                reply=True
            )


@chat.assign("del")
async def chat_del(arp: AlcResult, user: A_User):
    if arp.result.find("del.help"):
        await MessageFactory(Text(menus.get_func_string("chat.del"))).finish()
    del_params = arp.result.all_matched_args
    name = del_params.get("chat_name")
    public = arp.result.find("del.public")
    if not name:
        result = "命令有误:\n格式:chat del -n {chat_name} (-public)\n错误如下:\n参数 name 未填写\n\n如有疑问,请使用'chat help'查看图片版详细说明"
        await MessageFactory(Text(result)).finish(reply=True)
    if public:
        if user.user_id in common_config().superusers:
            user = public_user
        else:
            await MessageFactory(Text("你无权管理公有对话")).finish(reply=True)

    try:
        if name in user.bots.keys():
            del user.bots[name]
            user.save()
            await MessageFactory(
                Text(f"删除{'公有' if public else '私有'}会话 {name} 成功")
            ).finish(reply=True)
        else:
            await MessageFactory(
                Text(f"{'公有' if public else '私有'}会话 {name} 不存在")
            ).finish(reply=True)
    except FinishedException:
        ...
    except Exception as e:
        await MessageFactory(Text(f"删除{'公有' if public else '私有'}会话失败: {e}")).finish(
            reply=True
        )



from __future__ import annotations

import json
from pathlib import Path
from typing import Annotated, Tuple, Dict, Optional, Any  # noqa: F401

from nonebot import Bot
from nonebot.internal.params import Depends
from nonebot.params import Event
from nonebot_plugin_alconna import UniMsg, Reply
from nonebot_plugin_bind.utils import get_user
from pydantic import BaseModel

from .msgs_link import msg_links
from .web_config import common_config
from ..chatbots import chatbots
from ..utils import md_to_pic

Root = Path() / "data" / "SparkGPT"


class User(BaseModel):
    user_id: str
    image: Optional[bytes]
    bots_hash: Optional[int]
    bots: Dict[str, Any] = {}

    @property
    def store_path(self):
        return Root / "users" / f"{self.user_id}.json"

    def __init__(self, user_id: str, *args, **kwargs):
        super().__init__(user_id=user_id, *args, **kwargs)
        store_path = self.store_path
        if not store_path.exists():
            store_path.parent.mkdir(parents=True, exist_ok=True)
            self.save()
        else:
            with open(self.store_path, "r") as f:
                data = json.load(f)
            super().__init__(**data)
            for name, bot in self.bots.items():
                class_ = chatbots.get_model_class(bot["model"])
                self.bots[name] = class_(**bot)

    def save(self):
        self.bots = {
            key: value
            for key, value in sorted(self.bots.items(), key=lambda x: -len(x[0]))
        }
        copy = self.copy()
        copy.image = None
        copy.bots_hash = None
        data = copy.dict()
        with self.store_path.open("w", encoding="utf-8") as f:
            json.dump(data, f)

    async def get_bots_img(self):
        bots_hash = hash(str(self.bots) + str(public_user.bots))
        if bots_hash == self.bots_hash and self.image:
            return self.image
        else:
            private_bots = list(self.bots.values())
            if private_bots:
                md = f"私有会话\n使用' {common_config().private_command}名称 <问题> '即可使用相应会话进行对话  \n\n| 名称 | 模型 | 预设 | 指令 |  \n| -- | -- | -- | -- |  \n"
                for bot in private_bots:
                    md += f"| {bot.name} | {bot.model} | {bot.prompt_name} | {bot.command_name} |  \n"
            else:
                md = "私有会话:  \n\n你没有创建过私有会话"

            md += "  \n"
            public_bots = list(public_user.bots.values())
            if public_bots:
                md += f"公有会话\n使用' {common_config().public_command}名称 <问题> '即可使用相应会话进行对话  \n\n| 名称 | 模型 | 预设 | 指令 |  \n| -- | -- | -- | -- |  \n"
                for bot in public_bots:
                    md += f"| {bot.name} | {bot.model} | {bot.prompt_name} | {bot.command_name} |  \n"
            else:
                md += "公有会话:  \n\nSparkGPT管理员没有创建过公有会话"

            md += "  \n"

            img = await md_to_pic(md, width=400)
            self.image = img
            self.bots_hash = bots_hash
            return img


class Users:
    users: dict = {}

    def __getitem__(self, user_id: int | str) -> User:
        user_id = str(user_id)
        if user_id not in self.users:
            self.load_user(user_id)
        return self.users.get(user_id)

    def load_user(self, user_id: str):
        try:
            self.users[user_id] = User(user_id=user_id)
        except Exception as e:
            raise Exception("Failed to load user") from e


users = Users()
public_user = users["public_user"]



from functools import partial
from typing import Any, Dict, List, Literal, TypedDict, cast, Union, Optional


from nonebot.adapters import Bot as BaseBot
from nonebot.adapters import Event

from ..types import Text, Image, Reply, Mention
from ..utils import (
    Receipt,
    MessageFactory,
    PlatformTarget,
    SupportedAdapters,
    SupportedPlatform,
    MessageSegmentFactory,
    get_bot_id,
    register_sender,
    register_get_bot_id,
    register_ms_adapter,
    register_list_targets,
    register_convert_to_arg,
    assamble_message_factory,
    register_target_extractor,
)
from ..utils.platform_send_target import TargetKaiheilaChannel, TargetKaiheilaPrivate

try:
    from nonebot.adapters.kaiheila import Bot
    from nonebot.adapters.kaiheila.api import Guild, Channel, UserChat
    from nonebot.adapters.kaiheila.message import (
        Message,
        MessageSegment,
        MessageSerializer,
    )
    from nonebot.adapters.kaiheila.event import (
        MessageEvent,
        ChannelMessageEvent,
        PrivateMessageEvent,
    )


    def _unwrap_paging_api(field: str):
        def decorator(func):
            async def wrapper(*args, **kwargs):
                result = await func(*args, **kwargs)
                while True:
                    for x in getattr(result, field):
                        yield x

                    if result.meta.page != result.meta.page_total:
                        result = await func(*args, **kwargs, page=result.meta.page + 1)
                    else:
                        break

            return wrapper

        return decorator


    adapter = SupportedAdapters.kaiheila
    register_kaiheila = partial(register_ms_adapter, adapter)

    MessageFactory.register_adapter_message(SupportedAdapters.kaiheila, Message)


    @register_kaiheila(Text)
    def _text(t: Text) -> MessageSegment:
        return MessageSegment.text(t.data["text"])


    @register_kaiheila(Image)
    async def _image(i: Image, bot: BaseBot) -> MessageSegment:
        if not isinstance(bot, Bot):
            raise TypeError(f"Unsupported type of bot: {type(bot)}")

        file_key = await bot.upload_file(i.data["image"], i.data["name"])
        return MessageSegment.image(file_key)


    @register_kaiheila(Mention)
    def _mention(m: Mention) -> MessageSegment:
        return MessageSegment.KMarkdown("(met)" + m.data["user_id"] + "(met)")


    @register_kaiheila(Reply)
    def _reply(r: Reply) -> MessageSegment:
        return MessageSegment.quote(r.data["message_id"])


    @register_target_extractor(PrivateMessageEvent)
    def _extract_private_msg_event(event: Event) -> TargetKaiheilaPrivate:
        assert isinstance(event, PrivateMessageEvent)
        return TargetKaiheilaPrivate(user_id=event.user_id)


    @register_target_extractor(ChannelMessageEvent)
    def _extract_channel_msg_event(event: Event) -> TargetKaiheilaChannel:
        assert isinstance(event, ChannelMessageEvent)
        return TargetKaiheilaChannel(channel_id=event.target_id)


    @register_convert_to_arg(adapter, SupportedPlatform.kaiheila_private)
    def _gen_private(target: PlatformTarget) -> Dict[str, Any]:
        assert isinstance(target, TargetKaiheilaPrivate)
        return {
            "user_id": target.user_id,
        }


    @register_convert_to_arg(adapter, SupportedPlatform.kaiheila_channel)
    def _gen_channel(target: PlatformTarget) -> Dict[str, Any]:
        assert isinstance(target, TargetKaiheilaChannel)
        return {
            "channel_id": target.channel_id,
        }


    # https://developer.kookapp.cn/doc/http/message#%E5%8F%91%E9%80%81%E9%A2%91%E9%81%93%E8%81%8A%E5%A4%A9%E6%B6%88%E6%81%AF



    class RawResp(TypedDict):
        msg_id: str
        msg_timestamp: int
        nonce: str


    class KaiheilaReceipt(Receipt):
        sent_msg: RawResp
        message_id: str
        type: Literal["group", "private"]
        target_id: Union[str, int]
        reply_to_msg_id: Optional[Union[str, int]]
        mention_user_id: Optional[Union[str, int]]
        adapter_name = adapter

        async def revoke(self):
            if self.type == "group":
                return await cast(Bot, self._get_bot()).message_delete(
                    msg_id=self.message_id
                )
            else:
                return await cast(Bot, self._get_bot()).directMessage_delete(
                    msg_id=self.message_id
                )

        async def edit(self, msg, at_sender=False, reply=False):
            bot = self._get_bot()
            if at_sender:
                msg = Mention(user_id=self.mention_user_id) + msg
            if reply:
                msg = Reply(message_id=self.reply_to_msg_id) + msg

            message_to_send = Message()
            for message_segment_factory in msg:
                message_segment = await message_segment_factory.build(bot)
                message_to_send += message_segment

            params = {"msg_id": str(self.message_id)}
            # type & content
            if isinstance(message_to_send, Message):
                new_message = Message()
                # 提取message中的quote消息段
                for seg in message_to_send:
                    if seg.type == "quote":
                        params["quote"] = seg.data["msg_id"]
                    else:
                        new_message.append(seg)
                params["type"], params["content"] = MessageSerializer(
                    new_message
                ).serialize()
            elif isinstance(message_to_send, MessageSegment):
                params["type"], params["content"] = MessageSerializer(
                    Message(message_to_send)
                ).serialize()
            else:
                raise ValueError("message为空或有误")

            # target_id & api
            params["target_id"] = self.target_id
            if self.type == "group":
                api = "message/update"
            else:
                params["target_id"] = self.target_id
                api = "direct-message/update"

            return await bot.call_api(api, **params)

        @property
        def raw(self) -> RawResp:
            return self.sent_msg


    @register_sender(SupportedAdapters.kaiheila)
    async def send(
        bot,
        msg: MessageFactory[MessageSegmentFactory],
        target,
        event: Event,
        at_sender: bool,
        reply: bool,
    ) -> KaiheilaReceipt:
        assert isinstance(bot, Bot)
        assert isinstance(target, (TargetKaiheilaPrivate, TargetKaiheilaChannel))

        params = {}
        if event.get_event_name().startswith("message.group"):
            params["type"] = "group"
            params["target_id"] = target.channel_id
        else:
            params["type"] = "private"
            params["target_id"] = target.user_id

        params["mention_user_id"] = event.get_user_id()
        params["reply_to_msg_id"] = event.msg_id

        if event:
            assert isinstance(event, MessageEvent)
            full_msg = assamble_message_factory(
                msg,
                Mention(event.get_user_id()),
                Reply(event.msg_id),
                at_sender,
                reply,
            )
        else:
            full_msg = msg

        message_to_send = Message()
        for message_segment_factory in full_msg:
            message_segment = await message_segment_factory.build(bot)
            message_to_send += message_segment

        resp = await bot.send_msg(message=message_to_send, **target.arg_dict(bot))
        return KaiheilaReceipt(
            bot_id=get_bot_id(bot),
            sent_msg=cast(RawResp, resp),
            message_id=resp.msg_id,
            **params,
        )


    @register_list_targets(SupportedAdapters.kaiheila)
    async def list_targets(bot: BaseBot) -> List[PlatformTarget]:
        assert isinstance(bot, Bot)

        targets = []

        async for guild in _unwrap_paging_api("guilds")(bot.guild_list)():
            guild: Guild
            async for channel in _unwrap_paging_api("channels")(bot.channel_list)(
                guild_id=guild.id_
            ):
                assert isinstance(channel, Channel)
                assert channel.id_
                target = TargetKaiheilaChannel(channel_id=channel.id_)
                targets.append(target)

        async for user_chat in _unwrap_paging_api("user_chats")(bot.userChat_list)():
            user_chat: UserChat
            assert user_chat.target_info and user_chat.target_info.id_
            target = TargetKaiheilaPrivate(user_id=user_chat.target_info.id_)
            targets.append(target)

        return targets



    @register_get_bot_id(adapter)
    def _get_id(bot: BaseBot):
        assert isinstance(bot, Bot)
        return bot.self_id

except ImportError:
    pass
except Exception as e:
    raise e



from arclet.alconna import Alconna, Subcommand
from nonebot_plugin_alconna import on_alconna
from ..nonebot_plugin_saa import MessageFactory, Text, Image
from nonebot_plugin_spark_gpt.const import menus

from ..chatbots import chatbots
from ..utils import Command_Start

model = on_alconna(Alconna(f"{Command_Start}model",
                           Subcommand("help"),
                           Subcommand("list")))


@model.assign("$main")
async def mode_main():
    await MessageFactory(Text(menus.get_menu_string("model"))).finish()


@model.assign("help")
async def mode_help():
    await MessageFactory(Image(await menus.get_menu_image("model"))).finish()


@model.assign("list")
async def model_help():
    await MessageFactory(Image(await chatbots.get_image())).finish()



import asyncio
import json
from json import JSONDecodeError

import aiohttp
import tiktoken

from . import register_chatbot
from .BaseChatBot import BaseChatBot, Permission
from ..type_store.web_config import chatgptapi_config

LOCKS = {}
Api_Key = ""


class ChatGPTApiBot(BaseChatBot):
    messages = []

    @classmethod
    @property
    def able(self) -> bool:
        config = chatgptapi_config()
        return bool(config.api_key)

    async def refresh(self):
        self.ensure()
        async with LOCKS[self.__hash__()]:
            self.messages = [
                {
                    "role": "system",
                    "content": self.prompt
                    or "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally",
                }
            ]
            self.save()

    async def ask(self, question: str):
        async for text in self.base_ask(question):
            yield text

    async def base_ask(self, question: str):
        self.ensure()

        if (not self.messages) and self.prompt:
            await self.refresh()

        async with LOCKS[self.__hash__()]:
            question = self.process_question(question)

            self.messages.append({"role": "user", "content": question})

            self.__truncate_conversation()

            async with aiohttp.ClientSession() as session:
                async with session.post(
                    "https://api.openai.com/v1/chat/completions",
                    json={
                        "model": self.basemodel,
                        "messages": self.messages,
                        "stream": True,
                        "temperature": 0.5,
                        "top_p": 1.0,
                        "presence_penalty": 0.0,
                        "frequency_penalty": 0.0,
                        "n": 1,
                        "user": "user",
                        "max_tokens": self.max_token,
                    },
                    headers={"Authorization": f"Bearer {Api_Key}"},
                ) as resp:
                    if resp.status != 200:
                        raise Exception(
                            f"ChatGPTApi询问时出错: status:{resp.status}, reason: {await resp.text()}"
                        )
                    else:
                        full_content = ""
                        role = ""
                        async for data in resp.content:
                            data = data.decode("utf-8")[6:].rstrip("\n")
                            if data.startswith("{"):
                                try:
                                    data_json = json.loads(data)
                                    choices = data_json.get("choices")

                                    if not choices:
                                        continue

                                    delta = choices[0].get("delta")

                                    if not delta:
                                        continue

                                    role_ = delta.get("role")
                                    if role_:
                                        role = role_

                                    if "content" in delta:
                                        content = delta["content"]
                                        full_content += content
                                        yield content
                                except JSONDecodeError:
                                    pass

                        if not bool(role and content):
                            self.messages.pop()
                        else:
                            self.messages.append({"role": role, "content": content})
                            self.save()

    def __truncate_conversation(self):
        token_count = self.token_count
        token_limit = self.truncate_limit
        while (token_count > token_limit or (token_limit - token_count < 300)) and len(
            self.messages
        ) > 1:
            del self.messages[1]
        self.save()

    @property
    def max_token(self):
        if "gpt-4-32k" == self.basemodel:
            return 31000
        elif "gpt-4" == self.basemodel:
            return 7000
        elif "gpt-3.5-turbo-16k" == self.basemodel:
            return 15000
        else:
            return 4000

    @property
    def token_count(self) -> int:
        tiktoken.model.MODEL_TO_ENCODING["gpt-4"] = "cl100k_base"

        encoding = tiktoken.encoding_for_model(self.basemodel)

        num_tokens = 0
        for message in self.messages:
            # every message follows <im_start>{role/name}\n{content}<im_end>\n
            num_tokens += 5
            for key, value in message.items():
                if value:
                    num_tokens += len(encoding.encode(value))
                if key == "name":  # if there's a name, the role is omitted
                    num_tokens += 5  # role is always required and always 1 token
        num_tokens += 5  # every reply is primed with <im_start>assistant
        return num_tokens

    @property
    def truncate_limit(self) -> int:
        if "gpt-4-32k" == self.basemodel:
            return 30500
        elif "gpt-4" == self.basemodel:
            return 6500
        elif "gpt-3.5-turbo-16k" == self.basemodel:
            return 14500
        else:
            return 3500

    def ensure(self):
        global Api_Key, LOCKS
        config = chatgptapi_config()
        if not bool(config.api_key):
            raise Exception("ChatGPTApi模型已被禁用,目前无法使用")
        else:
            if Api_Key != config.api_key:
                Api_Key = config.api_key
        if self.__hash__() not in LOCKS.keys():
            LOCKS[self.__hash__()] = asyncio.Lock()


@register_chatbot
class GPT3_5ApiBot(ChatGPTApiBot):
    model = "GPT3_5Api"
    basemodel = "gpt-3.5-turbo"
    permission = Permission.common

    @classmethod
    @property
    def desc(self) -> str:
        return "ChatGPTApi的gpt 3.5 turbo模型"


@register_chatbot
class GPT_3_5_16kBot(ChatGPTApiBot):
    model = "GPT_3_5_16k"
    basemodel = "gpt-3.5-turbo-16k"
    permission = Permission.paid

    @classmethod
    @property
    def desc(self) -> str:
        return "ChatGPTApi的gpt 3.5 turbo 16K模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_3_5_turbo_16k and config.api_key)


@register_chatbot
class GPT_3_5_0301Bot(ChatGPTApiBot):
    model = "GPT_3_5_0301"
    basemodel = "gpt-3.5-turbo-0301"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 3.5 turbo 0301模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_3_5_turbo_0301 and config.api_key)


@register_chatbot
class GPT_3_5_0613Bot(ChatGPTApiBot):
    model = "GPT_3_5_0613"
    basemodel = "gpt-3.5-turbo-0613"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 3.5 turbo 0613模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_3_5_turbo_0613 and config.api_key)


@register_chatbot
class GPT_3_5_16k_0613Bot(ChatGPTApiBot):
    model = "GPT_3_5_16k_0613"
    basemodel = "gpt-3.5-turbo-16k-0613"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 3.5 turbo 16K 0613模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_3_5_turbo_16k_0613 and config.api_key)


@register_chatbot
class GPT_4Bot(ChatGPTApiBot):
    model = "GPT_4"
    basemodel = "gpt-4"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 4模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_4 and config.api_key)


@register_chatbot
class GPT_4_0314Bot(ChatGPTApiBot):
    model = "GPT_4_0314"
    basemodel = "gpt-4-0314"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 4 0314模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_4_0314 and config.api_key)


@register_chatbot
class GPT_4_32kBot(ChatGPTApiBot):
    model = "GPT_4_32k"
    basemodel = "gpt-4-32k"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 4 32K模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_4_32k and config.api_key)


@register_chatbot
class GPT_4_32k_0314Bot(ChatGPTApiBot):
    model = "GPT_4_32k_0314"
    basemodel = "gpt-4-32k-0314"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 4 32K 0314模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_4_32k_0314 and config.api_key)


@register_chatbot
class GPT_4_0613Bot(ChatGPTApiBot):
    model = "GPT_4_0613"
    basemodel = "gpt-4-0613"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 4 0613模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_4_0613 and config.api_key)


@register_chatbot
class GPT_4_32k_0613Bot(ChatGPTApiBot):
    model = "GPT_4_32k_0613"
    basemodel = "gpt-4-32k-0613"
    permission = Permission.paid

    @classmethod
    @property
    def desc(cls) -> str:
        return "ChatGPTApi的gpt 4 32K 0613模型"

    @classmethod
    @property
    def able(cls) -> bool:
        config = chatgptapi_config()
        return bool(config.enable_gpt_4_32k_0613 and config.api_key)



import random
import string


def generate_random(n):
    range_start = 10 ** (n - 1)
    range_end = (10 ** n) - 1
    numbers = random.randint(range_start, range_end)

    letters = ''.join(random.choice(string.ascii_letters) for i in range(n))

    result = str(numbers) + letters
    return result



from nonebot.plugin import PluginMetadata

from . import adapters as adapters
from .types import Custom as Custom
from .types import Image as Image
from .types import Mention as Mention
from .types import Reply as Reply
from .types import Text as Text
from .utils import AggregatedMessageFactory as AggregatedMessageFactory
from .utils import MessageFactory as MessageFactory
from .utils import MessageSegmentFactory as MessageSegmentFactory
from .utils import PlatformTarget as PlatformTarget
from .utils import SupportedAdapters as SupportedAdapters
from .utils import TargetDiscordChannel as TargetDiscordChannel
from .utils import TargetFeishuGroup as TargetFeishuGroup
from .utils import TargetFeishuPrivate as TargetFeishuPrivate
from .utils import TargetKaiheilaChannel as TargetKaiheilaChannel
from .utils import TargetKaiheilaPrivate as TargetKaiheilaPrivate
from .utils import TargetOB12Unknow as TargetOB12Unknow
from .utils import TargetQQGroup as TargetQQGroup
from .utils import TargetQQGuildChannel as TargetQQGuildChannel
from .utils import TargetQQGuildDirect as TargetQQGuildDirect
from .utils import TargetQQPrivate as TargetQQPrivate
from .utils import TargetTelegramCommon as TargetTelegramCommon
from .utils import TargetTelegramForum as TargetTelegramForum
from .utils import enable_auto_select_bot as enable_auto_select_bot
from .utils import extract_target as extract_target
from .utils import get_target as get_target

__plugin_meta__ = PluginMetadata(
    name="峯驰物流",
    description=("一个帮助处理不同 adapter 消息的适配和发送的插件 "),
    usage="请开发者参考文档",
    type="library",
    homepage="https://send-anything-anywhere.felinae98.cn/",
    supported_adapters={
        "~onebot.v11",
        "~onebot.v12",
        "~kaiheila",
        "~qqguild",
        "~telegram",
        "~feishu",
    },
)



import asyncio
from typing import AsyncGenerator

from . import register_chatbot
from .BaseChatBot import BaseChatBot, Permission
from ..type_store.web_config import claude_ai_config
from async_claude_client import ClaudeAiClient

config = claude_ai_config()
client: ClaudeAiClient = None
cookie: dict = {}
locks = {}


@register_chatbot
class Claude_AiBot(BaseChatBot):
    model = "Claude_Ai"
    permission = Permission.paid
    chat: dict = {}

    async def ensure(self):
        global client, cookie, locks
        if self.__hash__() not in locks.keys():
            locks[self.__hash__()] = asyncio.Lock()
        config = claude_ai_config()
        if not client or cookie != config.cookie:
            cookie = config.cookie
            client = await ClaudeAiClient(cookie).init()

    @classmethod
    @property
    def desc(self) -> str:
        return "Claude.ai网站的claude2模型, 有每日免费额度上限"

    @classmethod
    @property
    def able(cls) -> bool:
        config = claude_ai_config()
        return bool(config.cookie)

    async def refresh(self):
        await self.ensure()
        async with locks[self.__hash__()]:
            self.chat = await client.create_new_chat()
            self.save()
        if self.prompt:
            async for text in self.ask(self.prompt + "\n第一次回复只需要回复'我明白了',只需要回复'我明白了'"):
                pass

    async def ask(self, question: str) -> AsyncGenerator:
        await self.ensure()
        if not self.chat:
            await self.refresh()
        async with locks[self.__hash__()]:
            question = self.process_question(question)
            if not self.chat:
                await self.refresh()
            async for text in client.ask_stream(
                question=question, chat_id=self.chat["uuid"]
            ):
                yield text



import asyncio
import json
import uuid
from urllib.parse import urljoin

import aiohttp

from . import register_chatbot
from .BaseChatBot import BaseChatBot, Permission
from ..type_store.web_config import chatgptweb_config

AccessToken = ""
SESSION_TOKEN_KEY = "__Secure-next-auth.session-token"
SessionToken = chatgptweb_config().session_token
UA = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15"

LOCKS = {}


class ChatGPTWebBot(BaseChatBot):
    conversation_id: str = None
    parent_message_id: str = str(uuid.uuid4)

    @classmethod
    @property
    def able(cls):
        config = chatgptweb_config()
        return bool(config.api_url and config.session_token)

    async def ask(self, question):
        if (not self.conversation_id) and self.prompt:
            await self.refresh()
        else:
            await self.ensure()

        question = self.process_question(question)

        async with LOCKS[self.__hash__()]:
            async for text in self.base_ask(question):
                yield text
            self.save()

    async def refresh(self):
        await self.ensure()

        async with LOCKS[self.__hash__()]:
            self.conversation_id = None
            self.parent_message_id = str(uuid.uuid4())

            if self.prompt:
                async for text in self.base_ask(
                    self.prompt + "\n首次回复,请只回复'我知道了',请只回复'我知道了'"
                ):
                    pass
            self.save()

    async def ensure(self):
        global SessionToken, LOCKS
        config = chatgptweb_config()
        if not bool(config.session_token and config.api_url):
            raise Exception("模型 ChatGPTWeb 已被禁用,目前无法使用")
        session_token = config.session_token
        if (SessionToken != session_token) or not AccessToken:
            SessionToken = session_token
            await self.refresh_access_token()
        if self.__hash__() not in LOCKS.keys():
            LOCKS[self.__hash__()] = asyncio.Lock()

    async def base_ask(self, question: str):
        async with aiohttp.ClientSession(
            headers={"User-Agent": UA},
        ) as session:
            async with session.post(
                urljoin(chatgptweb_config().api_url, "backend-api/conversation"),
                headers=self.ask_header,
                json=self.build_ask_data(question=question),
                proxy=chatgptweb_config().proxy,
            ) as response:
                if response.status == 429:
                    msg = ""
                    _buffer = bytearray()
                    async for chunk in response.content.iter_any():
                        _buffer.extend(chunk)
                    resp = json.loads(_buffer.decode())
                    if detail := resp.get("detail"):
                        if isinstance(detail, str):
                            msg += "\n" + detail
                    raise Exception("请求过多,请放慢速度" + msg)

                if response.status == 401:
                    raise Exception("token失效,请重新设置token")

                if response.status == 403:
                    raise Exception("API错误,请联系开发者修复")

                if response.status == 404:
                    await self.refresh()
                    raise Exception("会话不存在")

                if response.status >= 400:
                    _buffer = bytearray()
                    async for chunk in response.content.iter_any():
                        _buffer.extend(chunk)
                    resp_text = _buffer.decode()
                    raise Exception(
                        f"ChatGPT 服务器返回了非预期的内容: HTTP{response.status}\n{resp_text[:256]}"
                    )
                saved = False
                last_text = ""
                yield_text = ""
                async for line in response.content:
                    if "internal server error" in str(line):
                        raise Exception("Internal Server Error: ChatGPT服务器内部错误")

                    try:
                        str_data = str(line).strip("b'data: ").rstrip("\\n")
                        if str_data.startswith("{"):
                            data = json.loads(
                                str_data.replace("\\\\", "\\"),
                            )
                            message = data.get("message", {})
                            if (message.get("author", {})).get("role") == "assistant":
                                if not saved:
                                    self.parent_message_id = message.get("id")
                                    self.conversation_id = data.get("conversation_id")
                                    saved = True
                                answer_text = message.get("content", {}).get(
                                    "parts", [""]
                                )[0]
                                yield_text = answer_text[len(last_text) :]
                                last_text = answer_text
                                yield yield_text
                        else:
                            continue
                    except:
                        pass
                if not saved:
                    raise Exception("获取ChatGPTWeb会话和回答失败")

    async def refresh_access_token(self):
        global AccessToken, SessionToken
        async with aiohttp.ClientSession(
            cookies={
                SESSION_TOKEN_KEY: SessionToken,
            },
            connector=aiohttp.TCPConnector(ssl=False),
        ) as session:
            async with session.get(
                urljoin(chatgptweb_config().api_url, "api/auth/session"),
                headers={
                    "User-Agent": UA,
                },
                proxy=chatgptweb_config().proxy,
            ) as response:
                try:
                    resp_json = await response.json()
                    SessionToken = (
                        response.cookies.get(SESSION_TOKEN_KEY) or SessionToken
                    )
                    AccessToken = resp_json["accessToken"]
                except Exception as e:
                    raise Exception(
                        f"ChatGPTWeb在刷新AccessToken时出错:\n 错误信息:{await response.text()} \n 错误原因:{e} "
                    )

    @property
    def ask_header(self):
        return {
            "Accept": "application/json",
            "Authorization": f"Bearer {AccessToken}",
            "Content-Type": "application/json",
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15",
            "X-Openai-Assistant-App-Id": "",
            "Connection": "close",
            "Accept-Language": "en-US,en;q=0.9",
            "Referer": "https://chat.openai.com/chat",
        }

    def build_ask_data(self, question: str):
        return {
            "action": "next",
            "messages": [
                {
                    "id": str(uuid.uuid4()),
                    "author": {"role": "user"},
                    "role": "user",
                    "content": {
                        "content_type": "text",
                        "parts": [question],
                    },
                }
            ],
            "conversation_id": self.conversation_id,
            "parent_message_id": self.parent_message_id,
            "model": self.basemodel,
            "timezone_offset_min": -480,
        }


@register_chatbot
class GPT35WebBot(ChatGPTWebBot):
    model = "GPT35Web"
    basemodel = "text-davinci-002-render-sha"
    permission = Permission.common

    @classmethod
    @property
    def desc(cls):
        return "ChatGPT Web网页版的默认模型,GPT3.5"


@register_chatbot
class GPT4WebBot(ChatGPTWebBot):
    model = "GPT4Web"
    basemodel = "gpt-4"
    permission = Permission.paid

    @classmethod
    @property
    def able(cls):
        config = chatgptweb_config()
        return bool(config.api_url and config.session_token and config.enable_GPT_4)

    @classmethod
    @property
    def desc(cls):
        return "ChatGPT Web网页版的付费订阅模型,只有订阅plus才可以使用"



import asyncio
from asyncio import ensure_future

from arclet.alconna import Alconna, Subcommand, Option, Args
from nonebot.exception import FinishedException
from nonebot_plugin_alconna import on_alconna, AlconnaMatch, Match, AlcResult
from ..nonebot_plugin_saa import MessageFactory, Image, Text

from ..const import menus
from ..type_store import prompt_config, common_config
from ..utils import Command_Start
from ..utils import get_url
from ..utils.extractor import A_User
prompt = on_alconna(
    Alconna(
        f"{Command_Start}prompt",
        Subcommand("help"),
        Subcommand("add", Option("-n", Args["name", str]), Option("-c", Args["content", str])),
        Subcommand("del", Option("-n", Args["name", str])),
        Subcommand("list", Option("-p", Args['page', int])),
        Subcommand("show", Option("-n", Args["name", str]))
    ),
    aliases={"预设"},
)


@prompt.assign("$main")
async def prompt_main():
    await MessageFactory(Text(menus.get_menu_string("prompt"))).finish()


@prompt.assign("help")
async def prompt_help():
    await MessageFactory(Image(await menus.get_menu_image("prompt"))).finish()


@prompt.assign("add")
async def prompt_add(arp: AlcResult, user: A_User):
    if user.user_id not in common_config().superusers:
        await MessageFactory(Text("无权管理预设")).finish(reply=True)
    prompt_params = {"name": "", "content": ""}
    prompt_params.update(arp.result.all_matched_args)
    result = "命令有误:\n格式:prompt add -n {name} -c {content}\n错误如下: "
    for key, value in prompt_params.items():
        if not value:
            result += f"\n参数{key}未填写"

    if not (prompt_params['name'] and prompt_params['content']):
        result += "\n\n如有疑问,请使用'prompt help'查看图片版详细说明"
        await MessageFactory(Text(result)).finish(reply=True)
    else:
        prompt_ = prompt_config()
        try:
            prompt_.data_dict[prompt_params["name"]] = prompt_params["content"]
            prompt_.save()
        except FinishedException:
            ...
        except Exception as e:
            await MessageFactory(Text(f"添加失败: {e}")).finish(reply=True)
        await MessageFactory(Text(f"成功添加新预设: {prompt_params['name']}")).finish(reply=True)


@prompt.assign("del")
async def prompt_del(user: A_User, name: Match[str] = AlconnaMatch("name")):
    if user.user_id not in common_config().superusers:
        await MessageFactory(Text("无权管理预设")).finish(reply=True)
    prompt_ = prompt_config()
    if name.available:
        if name.result in prompt_.data_dict.keys():
            del prompt_.data_dict[name.result]
            prompt_.save()
            await MessageFactory(Text(f"成功删除了预设: {name.result}")).finish(reply=True)
        else:
            await MessageFactory(Text("不存在该名称的预设,请先使用 'prompt list -p {num}'查询可用预设列表")).finish(
                reply=True)
    else:
        await MessageFactory(
            Text(
                "命令有误:\n格式:prompt del -n {name}\n错误如下:参数name未填写,请补全后重新发送\n如有疑问,请使用'prompt help'查看图片版详细说明")).finish(
            reply=True)


@prompt.assign("list")
async def prompt_list_(page: Match[int] = AlconnaMatch('page')):
    if page.available:
        page = page.result - 1
    else:
        page = 0
    images = await prompt_config().get_images()
    await MessageFactory(Image(images[page])).finish()


@prompt.assign("show")
async def prompt_show(name: Match[str] = AlconnaMatch("name")):
    prompt_ = prompt_config()
    if name.available:
        if name.result in prompt_.data_dict.keys():
            image_bytes = ensure_future(prompt_.get_image(name.result))
            url = ensure_future(get_url(prompt_.data_dict.get(name.result)))
            await asyncio.gather(image_bytes, url)
            await MessageFactory(Image(image_bytes.result()) + Text(url.result())).finish(reply=True)  # noqa: E501
        else:
            await MessageFactory(Text("不存在该名称的预设,请先使用 'prompt list -p {num}'查询可用预设列表")).finish(
                reply=True)
    else:
        await MessageFactory(
            Text(
                "命令有误:\n格式:prompt show -n {name}\n错误如下:参数name未填写,请补全后重新发送\n如有疑问,请使用'prompt help'查看图片版详细说明")).finish(
            reply=True)



from functools import partial
from typing import Any, Dict, List, Union, Optional, cast


from nonebot.adapters import Bot, Event

from ..types import Text, Image, Reply, Mention
from ..utils import (
    Receipt,
    TargetQQGroup,
    MessageFactory,
    PlatformTarget,
    TargetQQPrivate,
    SupportedAdapters,
    SupportedPlatform,
    MessageSegmentFactory,
    AggregatedMessageFactory,
    get_bot_id,
    register_sender,
    register_get_bot_id,
    register_ms_adapter,
    register_list_targets,
    register_convert_to_arg,
    assamble_message_factory,
    register_target_extractor,
)

try:
    from nonebot.adapters.onebot.v11 import Bot as BotOB11
    from nonebot.adapters.onebot.v11.message import Message, MessageSegment
    from nonebot.adapters.onebot.v11 import (
        MessageEvent,
        PokeNotifyEvent,
        HonorNotifyEvent,
        GroupMessageEvent,
        GroupRequestEvent,
        FriendRequestEvent,
        GroupBanNoticeEvent,
        PrivateMessageEvent,
        FriendAddNoticeEvent,
        LuckyKingNotifyEvent,
        GroupAdminNoticeEvent,
        GroupRecallNoticeEvent,
        GroupUploadNoticeEvent,
        FriendRecallNoticeEvent,
        GroupDecreaseNoticeEvent,
        GroupIncreaseNoticeEvent,
    )

    adapter = SupportedAdapters.onebot_v11
    register_onebot_v11 = partial(register_ms_adapter, adapter)

    MessageFactory.register_adapter_message(SupportedAdapters.onebot_v11, Message)


    @register_onebot_v11(Text)
    def _text(t: Text) -> MessageSegment:
        return MessageSegment.text(t.data["text"])


    @register_onebot_v11(Image)
    async def _image(i: Image) -> MessageSegment:
        return MessageSegment.image(i.data["image"])


    @register_onebot_v11(Mention)
    async def _mention(m: Mention) -> MessageSegment:
        return MessageSegment.at(m.data["user_id"])


    @register_onebot_v11(Reply)
    async def _reply(r: Reply) -> MessageSegment:
        return MessageSegment.reply(int(r.data["message_id"]))


    @register_target_extractor(PrivateMessageEvent)
    def _extract_private_msg_event(event: Event) -> TargetQQPrivate:
        assert isinstance(event, PrivateMessageEvent)
        return TargetQQPrivate(user_id=event.user_id)


    @register_target_extractor(GroupMessageEvent)
    def _extract_group_msg_event(event: Event) -> TargetQQGroup:
        assert isinstance(event, GroupMessageEvent)
        return TargetQQGroup(group_id=event.group_id)


    @register_target_extractor(FriendAddNoticeEvent)
    @register_target_extractor(FriendRecallNoticeEvent)
    def _extract_friend_notice_event(event: Event) -> TargetQQPrivate:
        assert isinstance(event, (FriendAddNoticeEvent, FriendRecallNoticeEvent))
        return TargetQQPrivate(user_id=event.user_id)


    @register_target_extractor(GroupBanNoticeEvent)
    @register_target_extractor(GroupAdminNoticeEvent)
    @register_target_extractor(GroupRecallNoticeEvent)
    @register_target_extractor(GroupUploadNoticeEvent)
    @register_target_extractor(GroupDecreaseNoticeEvent)
    @register_target_extractor(GroupIncreaseNoticeEvent)
    def _extract_group_notice_event(event: Event) -> TargetQQGroup:
        assert isinstance(
            event,
            (
                GroupBanNoticeEvent,
                GroupAdminNoticeEvent,
                GroupRecallNoticeEvent,
                GroupUploadNoticeEvent,
                GroupDecreaseNoticeEvent,
                GroupIncreaseNoticeEvent,
            ),
        )
        return TargetQQGroup(group_id=event.group_id)


    @register_target_extractor(HonorNotifyEvent)
    @register_target_extractor(LuckyKingNotifyEvent)
    def _extract_group_notify_event(event: Event) -> TargetQQGroup:
        assert isinstance(event, (HonorNotifyEvent, LuckyKingNotifyEvent))
        return TargetQQGroup(group_id=event.group_id)


    @register_target_extractor(FriendRequestEvent)
    def _extract_friend_request_event(event: Event) -> TargetQQPrivate:
        assert isinstance(event, FriendRequestEvent)
        return TargetQQPrivate(user_id=event.user_id)


    @register_target_extractor(GroupRequestEvent)
    def _extract_group_request_event(event: Event) -> TargetQQGroup:
        assert isinstance(event, GroupRequestEvent)
        return TargetQQGroup(group_id=event.group_id)


    @register_target_extractor(PokeNotifyEvent)
    def _extract_poke_notify_event(
        event: Event,
    ) -> Union[TargetQQPrivate, TargetQQGroup]:
        assert isinstance(event, PokeNotifyEvent)
        if event.group_id is not None:
            return TargetQQGroup(group_id=event.group_id)
        else:
            return TargetQQPrivate(user_id=event.user_id)


    @register_convert_to_arg(adapter, SupportedPlatform.qq_private)
    def _gen_private(target: PlatformTarget) -> Dict[str, Any]:
        assert isinstance(target, TargetQQPrivate)
        return {
            "message_type": "private",
            "user_id": target.user_id,
        }


    @register_convert_to_arg(adapter, SupportedPlatform.qq_group)
    def _gen_group(target: PlatformTarget) -> Dict[str, Any]:
        assert isinstance(target, TargetQQGroup)
        return {
            "message_type": "group",
            "group_id": target.group_id,
        }


    class OB11Receipt(Receipt):
        message_id: int
        adapter_name = adapter


        async def revoke(self):
            return await cast(BotOB11, self._get_bot()).delete_msg(
                message_id=self.message_id
            )

        @property
        def raw(self) -> Any:
            return self.message_id


    @register_sender(SupportedAdapters.onebot_v11)
    async def send(
        bot,
        msg: MessageFactory[MessageSegmentFactory],
        target,
        event,
        at_sender: bool,
        reply: bool,
    ) -> OB11Receipt:
        assert isinstance(bot, BotOB11)
        assert isinstance(target, (TargetQQGroup, TargetQQPrivate))
        if event:
            assert isinstance(event, MessageEvent)
            full_msg = assamble_message_factory(
                msg,
                Mention(event.get_user_id()),
                Reply(event.message_id),
                at_sender,
                reply,
            )
        else:
            full_msg = msg
        message_to_send = Message()
        for message_segment_factory in full_msg:
            message_segment = await message_segment_factory.build(bot)
            message_to_send += message_segment
        # https://github.com/botuniverse/onebot-11/blob/master/api/public.md#send_msg-%E5%8F%91%E9%80%81%E6%B6%88%E6%81%AF
        res_dict = await bot.send_msg(message=message_to_send, **target.arg_dict(bot))
        message_id = cast(int, res_dict["message_id"])
        return OB11Receipt(bot_id=get_bot_id(bot), message_id=message_id)



    @AggregatedMessageFactory.register_aggregated_sender(adapter)
    async def aggregate_send(
        bot: Bot,
        message_factories: List[MessageFactory],
        target: PlatformTarget,
        event: Optional[Event],
    ):
        assert isinstance(bot, BotOB11)
        login_info = await bot.get_login_info()

        msg_list: List[Message] = []
        for msg_fac in message_factories:
            msg = await msg_fac.build(bot)
            assert isinstance(msg, Message)
            msg_list.append(msg)
        aggregated_message_segment = Message(
            [
                MessageSegment.node_custom(
                    user_id=login_info["user_id"],
                    nickname=login_info["nickname"],
                    content=msg,
                )
                for msg in msg_list
            ]
        )

        if isinstance(target, TargetQQGroup):
            await bot.send_group_forward_msg(
                group_id=target.group_id, messages=aggregated_message_segment
            )
        elif isinstance(target, TargetQQPrivate):
            await bot.send_private_forward_msg(
                user_id=target.user_id, messages=aggregated_message_segment
            )
        else:  # pragma: no cover
            raise RuntimeError(f"{target.__class__.__name__} not supported")


    @register_list_targets(SupportedAdapters.onebot_v11)
    async def list_targets(bot: Bot) -> List[PlatformTarget]:
        assert isinstance(bot, BotOB11)

        targets = []
        groups = await bot.get_group_list()
        for group in groups:
            group_id = group["group_id"]
            target = TargetQQGroup(group_id=group_id)
            targets.append(target)

        # 获取好友列表
        users = await bot.get_friend_list()
        for user in users:
            user_id = user["user_id"]
            target = TargetQQPrivate(user_id=user_id)
            targets.append(target)

        return targets



    @register_get_bot_id(adapter)
    def _get_bot_id(bot: Bot):
        assert isinstance(bot, BotOB11)
        return bot.self_id

except ImportError:
    pass
except Exception as e:
    raise e



from .auto_select_bot import enable_auto_select_bot as enable_auto_select_bot
from .auto_select_bot import get_bot_id as get_bot_id
from .auto_select_bot import register_get_bot_id as register_get_bot_id
from .auto_select_bot import register_list_targets as register_list_targets
from .const import SupportedAdapters as SupportedAdapters
from .const import SupportedPlatform as SupportedPlatform
from .const import supported_adapter_names as supported_adapter_names
from .exceptions import AdapterNotInstalled as AdapterNotInstalled
from .exceptions import AdapterNotSupported as AdapterNotSupported
from .helpers import extract_adapter_type as extract_adapter_type
from .platform_send_target import (
    AllSupportedPlatformTarget as AllSupportedPlatformTarget,
)
from .platform_send_target import PlatformTarget as PlatformTarget
from .platform_send_target import QQGuildDMSManager as QQGuildDMSManager
from .platform_send_target import TargetDiscordChannel as TargetDiscordChannel
from .platform_send_target import TargetFeishuGroup as TargetFeishuGroup
from .platform_send_target import TargetFeishuPrivate as TargetFeishuPrivate
from .platform_send_target import TargetKaiheilaChannel as TargetKaiheilaChannel
from .platform_send_target import TargetKaiheilaPrivate as TargetKaiheilaPrivate
from .platform_send_target import TargetOB12Unknow as TargetOB12Unknow
from .platform_send_target import TargetQQGroup as TargetQQGroup
from .platform_send_target import TargetQQGuildChannel as TargetQQGuildChannel
from .platform_send_target import TargetQQGuildDirect as TargetQQGuildDirect
from .platform_send_target import TargetQQPrivate as TargetQQPrivate
from .platform_send_target import TargetTelegramCommon as TargetTelegramCommon
from .platform_send_target import TargetTelegramForum as TargetTelegramForum
from .platform_send_target import extract_target as extract_target
from .platform_send_target import get_target as get_target
from .platform_send_target import register_convert_to_arg as register_convert_to_arg
from .platform_send_target import register_qqguild_dms as register_qqguild_dms
from .platform_send_target import register_sender as register_sender
from .platform_send_target import register_target_extractor as register_target_extractor
from .receipt import Receipt as Receipt
from .types import AggregatedMessageFactory as AggregatedMessageFactory
from .types import BuildFunc as BuildFunc
from .types import CustomBuildFunc as CustomBuildFunc
from .types import MessageFactory as MessageFactory
from .types import MessageSegmentFactory as MessageSegmentFactory
from .types import assamble_message_factory as assamble_message_factory
from .types import do_build as do_build
from .types import do_build_custom as do_build_custom
from .types import register_ms_adapter as register_ms_adapter



from __future__ import annotations

from typing import Any, Dict

from pydantic import BaseModel


class MsgLinks(BaseModel):
    links: Dict[str, Dict[str | int, Any]] = {}

    def get_chatbot(self, user_id: str, msg_id: str | int):
        msg_id = str(msg_id)
        if user_id not in self.links:
            raise Exception("MsgLink Not Found")
        if msg_id in self.links[user_id]:
            self.clean(user_id)
            return self.links[user_id][msg_id]
        else:
            raise Exception("MsgLink Not Found")

    def add_msglink(self, user_id: str, msg_id: str | int, chatbot: Any):
        if user_id not in self.links.keys():
            self.links[user_id] = {}
        self.clean(user_id)
        self.links[user_id][str(msg_id)] = chatbot

    def clean(self, user_id: str):
        if len(self.links[user_id]) > 1000:
            self.links[user_id] = {}


msg_links = MsgLinks()



import asyncio
from asyncio import ensure_future

from arclet.alconna import Alconna, Subcommand, Option, Args
from nonebot.exception import FinishedException
from nonebot_plugin_alconna import on_alconna, AlconnaMatch, Match, AlcResult
from ..nonebot_plugin_saa import MessageFactory, Image, Text

from ..const import menus
from ..type_store import command_config, common_config
from ..utils import Command_Start
from ..utils import get_url
from ..utils.extractor import A_User
command = on_alconna(
    Alconna(f"{Command_Start}command",
            Subcommand("help"),
            Subcommand(
                "add", Option("-n", Args["name", str]), Option("-c", Args["content", str])
            ),  # noqa: E501
            Subcommand("del", Option("-n", Args["name", str])),
            Subcommand("list", Option("-p", Args["page", int])),
            Subcommand("show", Option("-n", Args["name", str])),
            ),
    aliases={"指令"},
)


@command.assign("$main")
async def command_main():
    await MessageFactory(Text(menus.get_menu_string("command"))).finish()


@command.assign("help")
async def command_help():
    await MessageFactory(Image(await menus.get_menu_image("command"))).finish()


@command.assign("add")
async def command_add(arp: AlcResult, user: A_User):
    if user.user_id not in common_config().superusers:
        await MessageFactory(Text("无权管理指令")).finish(reply=True)
    command_params = {"name": "", "content": ""}
    command_params.update(arp.result.all_matched_args)
    result = "命令有误:\n格式:command add -n {name} -c {content}\n错误如下: "
    for key, value in command_params.items():
        if not value:
            result += f"\n参数{key}未填写"

    if not (command_params["name"] and command_params["content"]):
        result += "\n\n如有疑问,请使用'command help'查看图片版详细说明"
        await MessageFactory(Text(result)).finish(reply=True)
    else:
        command_ = command_config()
        try:
            command_.data_dict[command_params["name"]] = command_params["content"]
            command_.save()
        except FinishedException:
            ...
        except Exception as e:
            await MessageFactory(Text(f"添加失败: {e}")).finish(reply=True)
        await MessageFactory(Text(f"成功添加新指令: {command_params['name']}")).finish(
            reply=True
        )


@command.assign("del")
async def command_del(user: A_User, name: Match[str] = AlconnaMatch("name")):
    if user.user_id not in common_config().superusers:
        await MessageFactory(Text("无权管理指令")).finish(reply=True)
    command_ = command_config()
    if name.available:
        if name.result in command_.data_dict.keys():
            del command_.data_dict[name.result]
            command_.save()
            await MessageFactory(Text(f"成功删除了指令: {name.result}")).finish(reply=True)
        else:
            await MessageFactory(
                Text("不存在该名称的指令,请先使用 'command list -p {num}'查询可用指令列表")
            ).finish(reply=True)
    else:
        await MessageFactory(
            Text(
                "命令有误:\n格式:command del -n {name}\n错误如下:参数name未填写,请补全后重新发送\n如有疑问,请使用'command help'查看图片版详细说明"
            )
        ).finish(reply=True)


@command.assign("list")
async def command_list_(page: Match[int] = AlconnaMatch("page")):
    if page.available:
        page = page.result - 1
    else:
        page = 0
    images = await command_config().get_images()
    await MessageFactory(Image(images[page])).finish()


@command.assign("show")
async def command_show(name: Match[str] = AlconnaMatch("name")):
    command_ = command_config()
    if name.available:
        if name.result in command_.data_dict.keys():
            image_bytes = ensure_future(command_.get_image(name.result))
            url = ensure_future(get_url(command_.data_dict.get(name.result)))
            await asyncio.gather(image_bytes, url)
            await MessageFactory(
                Image(image_bytes.result()) + Text(url.result())
            ).finish(
                reply=True
            )  # noqa: E501
        else:
            await MessageFactory(
                Text("不存在该名称的指令,请先使用 'command list -p {num}'查询可用指令列表")
            ).finish(reply=True)
    else:
        await MessageFactory(
            Text(
                "命令有误:\n格式:command show -n {name}\n错误如下:参数name未填写,请补全后重新发送\n如有疑问,请使用'command help'查看图片版详细说明"
            )
        ).finish(reply=True)



import json
from abc import ABC
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    Type,
    Tuple,
    Union,
    Literal,
    Callable,
    ClassVar,
    Optional,
    Awaitable,
    cast,
)

from nonebot.adapters import Bot, Event
from pydantic import BaseModel

from .const import SupportedAdapters, SupportedPlatform
from .helpers import extract_adapter_type

if TYPE_CHECKING:
    from .receipt import Receipt
    from .types import MessageFactory



class PlatformTarget(BaseModel, ABC):
    _deseriazer_map: ClassVar[
        Dict[SupportedPlatform, Type["AllSupportedPlatformTarget"]]
    ] = {}
    platform_type: SupportedPlatform

    class Config:
        frozen = True
        orm_mode = True

    def __init_subclass__(cls) -> None:
        assert isinstance(cls.__fields__["platform_type"].default, SupportedPlatform)
        cls = cast(Type["AllSupportedPlatformTarget"], cls)
        cls._deseriazer_map[cls.__fields__["platform_type"].default] = cls
        return super().__init_subclass__()

    def arg_dict(self, bot: Bot):
        adapter_type = extract_adapter_type(bot)
        if (self.platform_type, adapter_type) not in convert_to_arg_map.keys():
            raise RuntimeError(
                f"PlatformTarget {self.platform_type} not support {adapter_type}",
            )
        return convert_to_arg_map[(self.platform_type, adapter_type)](self)

    @classmethod
    def deserialize(cls, source: Any) -> "AllSupportedPlatformTarget":
        if isinstance(source, str):
            raw_obj = json.loads(source)
        else:
            raw_obj = source
            assert raw_obj.get("platform_type")
        platform_type = cast(
            SupportedPlatform, SupportedPlatform(raw_obj["platform_type"])
        )
        return cls._deseriazer_map[platform_type].parse_obj(raw_obj)


class TargetQQGroup(PlatformTarget):
    """QQ群

    参数
        group_id: 群号
    """

    platform_type: Literal[SupportedPlatform.qq_group] = SupportedPlatform.qq_group
    group_id: int


class TargetQQPrivate(PlatformTarget):
    """QQ私聊

    参数
        user_id: QQ号
    """

    platform_type: Literal[SupportedPlatform.qq_private] = SupportedPlatform.qq_private
    user_id: int


class TargetQQGuildChannel(PlatformTarget):
    """QQ频道子频道

    参数
        channel_id: 子频道号
    """

    platform_type: Literal[
        SupportedPlatform.qq_guild_channel
    ] = SupportedPlatform.qq_guild_channel
    channel_id: int


class TargetQQGuildDirect(PlatformTarget):
    """QQ频道私聊

    参数
        recipient_id: 接收人ID
        source_guild_id: 来自的频道号
    """

    platform_type: Literal[
        SupportedPlatform.qq_guild_direct
    ] = SupportedPlatform.qq_guild_direct
    recipient_id: int
    source_guild_id: int


class TargetOB12Unknow(PlatformTarget):
    """暂未识别的 Onebot v12 发送目标

    参数
        detail_type: "private" or "group" or "channel"
        user_id, group_id, channel_id, guild_id: 同 ob12 定义
    """

    platform_type: Literal[
        SupportedPlatform.unknown_ob12
    ] = SupportedPlatform.unknown_ob12
    platform: str
    detail_type: Literal["private", "group", "channel"]
    user_id: Optional[str] = None
    group_id: Optional[str] = None
    guild_id: Optional[str] = None
    channel_id: Optional[str] = None


class TargetKaiheilaChannel(PlatformTarget):
    """开黑啦频道

    参数
        channel_id: 频道ID
    """

    platform_type: Literal[
        SupportedPlatform.kaiheila_channel
    ] = SupportedPlatform.kaiheila_channel
    channel_id: str


class TargetKaiheilaPrivate(PlatformTarget):
    """开黑啦私聊

    参数
        user_id: 接收人ID
    """

    platform_type: Literal[
        SupportedPlatform.kaiheila_private
    ] = SupportedPlatform.kaiheila_private
    user_id: str


class TargetTelegramCommon(PlatformTarget):
    """Telegram 普通对话

    参数
        user_id: 对话ID
    """

    platform_type: Literal[
        SupportedPlatform.telegram_common
    ] = SupportedPlatform.telegram_common
    chat_id: Union[int, str]


class TargetTelegramForum(PlatformTarget):
    """Telegram 频道

    参数
        chat_id: 频道ID
        message_thread_id: 板块ID
    """

    platform_type: Literal[
        SupportedPlatform.telegram_forum
    ] = SupportedPlatform.telegram_forum
    chat_id: int
    message_thread_id: int


class TargetFeishuPrivate(PlatformTarget):
    """飞书私聊

    参数
        open_id: 用户 Open ID
    """

    platform_type: Literal[
        SupportedPlatform.feishu_private
    ] = SupportedPlatform.feishu_private
    open_id: str


class TargetFeishuGroup(PlatformTarget):
    """飞书群聊

    参数
        chat_id: 群 ID
    """

    platform_type: Literal[
        SupportedPlatform.feishu_group
    ] = SupportedPlatform.feishu_group
    chat_id: str


class TargetDiscordChannel(PlatformTarget):
    """Discord频道,包括群聊和私聊

    参数
        channel_id: 频道 ID
    """

    platform_type: Literal[
        SupportedPlatform.discord_channel
    ] = SupportedPlatform.discord_channel
    channel_id: int


# this union type is for deserialize pydantic model with nested PlatformTarget
AllSupportedPlatformTarget = Union[
    TargetQQGroup,
    TargetQQPrivate,
    TargetQQGuildChannel,
    TargetQQGuildDirect,
    TargetKaiheilaPrivate,
    TargetKaiheilaChannel,
    TargetOB12Unknow,
    TargetTelegramCommon,
    TargetTelegramForum,
    TargetFeishuPrivate,
    TargetFeishuGroup,
    TargetDiscordChannel
]

ConvertToArg = Callable[[PlatformTarget], Dict[str, Any]]
convert_to_arg_map: Dict[Tuple[SupportedPlatform, SupportedAdapters], ConvertToArg] = {}


def register_convert_to_arg(adapter: SupportedAdapters, platform: SupportedPlatform):
    def wrapper(func: ConvertToArg):
        convert_to_arg_map[(platform, adapter)] = func
        return func

    return wrapper


Extractor = Callable[[Event], PlatformTarget]
extractor_map: Dict[Type[Event], Extractor] = {}


def register_target_extractor(event: Type[Event]):
    def wrapper(func: Extractor):
        extractor_map[event] = func
        return func

    return wrapper


def extract_target(event: Event) -> PlatformTarget:
    """从事件中提取出发送目标，如果不能提取就抛出错误"""
    for event_type in event.__class__.mro():
        if event_type in extractor_map:
            if not issubclass(event_type, Event):
                break
            return extractor_map[event_type](event)
    raise RuntimeError(f"event {event.__class__} not supported")


def get_target(event: Event) -> Optional[PlatformTarget]:
    """从事件中提取出发送目标，如果不能提取就返回 None"""
    try:
        return extract_target(event)
    except RuntimeError:
        pass


Sender = Callable[
    [Bot, "MessageFactory", "PlatformTarget", Optional[Event], bool, bool],
    Awaitable["Receipt"],
]

sender_map: Dict[SupportedAdapters, Sender] = {}


def register_sender(adapter: SupportedAdapters):
    def wrapper(sender: Sender):
        sender_map[adapter] = sender
        return sender

    return wrapper


QQGuild_DMS = Callable[[TargetQQGuildDirect, Bot], Awaitable[int]]
qqguild_dms_map: Dict[SupportedAdapters, QQGuild_DMS] = {}


def register_qqguild_dms(adapter: SupportedAdapters):
    def wrapper(func: QQGuild_DMS):
        qqguild_dms_map[adapter] = func
        return func

    return wrapper


class QQGuildDMSManager:
    _cache: ClassVar[Dict[TargetQQGuildDirect, int]] = {}

    @classmethod
    def get_guild_id(cls, target: TargetQQGuildDirect) -> int:
        """从缓存中获取私聊所需 guild_id"""
        return cls._cache[target]

    @classmethod
    async def aget_guild_id(cls, target: TargetQQGuildDirect, bot: Bot) -> int:
        """获取私聊所需 guild_id"""
        if target in cls._cache:
            return cls._cache[target]

        adapter = extract_adapter_type(bot)
        if not (qqguild_dms := qqguild_dms_map.get(adapter)):
            raise RuntimeError(
                f"qqguild dms method for {adapter} not registered",
            )  # pragma: no cover
        guild_id = await qqguild_dms(target, bot)
        cls._cache[target] = guild_id
        return guild_id



from . import discord as discord
from . import feishu as feishu
from . import kaiheila as kaiheila
from . import onebot_v11 as onebot_v11
from . import onebot_v12 as onebot_v12
from . import qqguild as qqguild
from . import telegram as telegram



import asyncio

import aiohttp

from .decorators import retry
from .generate import generate_random

csrf_token = generate_random(32)
session_id = generate_random(32)
HEADERS = {
    "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
    "accept-language": "zh-CN,zh;q=0.9",
    "cache-control": "no-cache",
    "content-type": "application/x-www-form-urlencoded",
    "pragma": "no-cache",
    "sec-ch-ua": "\"Chromium\";v=\"116\", \"Not)A;Brand\";v=\"24\", \"Google Chrome\";v=\"116\"",
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": "\"Windows\"",
    "sec-fetch-dest": "document",
    "sec-fetch-mode": "navigate",
    "sec-fetch-site": "same-origin",
    "sec-fetch-user": "?1",
    "upgrade-insecure-requests": "1",
    "cookie": f"csrftoken={csrf_token}; sessionid={session_id}",
    "Referer": "https://dpaste.org/",
    "Referrer-Policy": "same-origin"
}


async def get_url(content: str, format_: str = "_markdown"):
    @retry(5, "粘贴到pastebin时出错")
    async def get_paste_url(content_: str, format__: str = "_markdown"):
        async with aiohttp.ClientSession(headers=HEADERS) as session:
            payload = {
                "csrfmiddlewaretoken": csrf_token,
                "title": "",
                "expires": "2592000",
                "format": "url",
                "lexer": format__,
                "content": content_,
            }
            async with session.post(
                    "https://dpaste.org", data=payload
            ) as resp:
                resp.raise_for_status()
                url = resp.url
                return str(url)

    try:
        url = await get_paste_url(content, format_)
        return url
    except Exception as e:
        return str(e)


if __name__ == '__main__':
    async def main():
        url = await get_url("1234")
        print(url)


    asyncio.run(main())



import asyncio
from abc import ABC
from copy import deepcopy
from inspect import signature
from typing import (
    Dict,
    List,
    Type,
    Union,
    TypeVar,
    Callable,
    ClassVar,
    Iterable,
    NoReturn,
    Optional,
    Awaitable,
    cast,
)
from warnings import warn

from nonebot.adapters import Bot, Event, Message, MessageSegment
from nonebot.exception import PausedException, FinishedException, RejectedException
from nonebot.matcher import current_bot, current_event, current_matcher
from typing_extensions import Self

from .receipt import Receipt
from .auto_select_bot import get_bot
from .const import SupportedAdapters
from .exceptions import FallbackToDefault, AdapterNotInstalled
from .helpers import extract_adapter_type
from .platform_send_target import PlatformTarget, sender_map, extract_target
from .receipt import Receipt

TMSF = TypeVar("TMSF", bound="MessageSegmentFactory")
TMF = TypeVar("TMF", bound="MessageFactory")
BuildFunc = Union[
    Callable[[TMSF], Union[MessageSegment, Awaitable[MessageSegment]]],
    Callable[[TMSF, Bot], Union[MessageSegment, Awaitable[MessageSegment]]],
]
CustomBuildFunc = Union[
    Callable[[], Union[MessageSegment, Awaitable[MessageSegment]]],
    Callable[[Bot], Union[MessageSegment, Awaitable[MessageSegment]]],
]


async def do_build(
    msf: "MessageSegmentFactory",
    builder: BuildFunc,
    bot: Bot,
) -> MessageSegment:
    if len(signature(builder).parameters) == 1:
        builder = cast(
            Callable[
                ["MessageSegmentFactory"],
                Union[MessageSegment, Awaitable[MessageSegment]],
            ],
            builder,
        )
        res = builder(msf)
    elif len(signature(builder).parameters) == 2:
        builder = cast(
            Callable[
                ["MessageSegmentFactory", Bot],
                Union[MessageSegment, Awaitable[MessageSegment]],
            ],
            builder,
        )
        res = builder(msf, bot)
    else:
        raise RuntimeError
    if asyncio.iscoroutine(res):
        return await res
    return cast(MessageSegment, res)


async def do_build_custom(builder: CustomBuildFunc, bot: Bot) -> MessageSegment:
    if len(signature(builder).parameters) == 0:
        builder = cast(
            Callable[[], Union[MessageSegment, Awaitable[MessageSegment]]],
            builder,
        )
        res = builder()
    elif len(signature(builder).parameters) == 1:
        builder = cast(
            Callable[[Bot], Union[MessageSegment, Awaitable[MessageSegment]]],
            builder,
        )
        res = builder(bot)
    else:
        raise RuntimeError
    if asyncio.iscoroutine(res):
        return await res
    return cast(MessageSegment, res)


class MessageSegmentFactory(ABC):
    _builders: ClassVar[
        Dict[
            SupportedAdapters,
            Union[
                Callable[[Self], Union[MessageSegment, Awaitable[MessageSegment]]],
                Callable[[Self, Bot], Union[MessageSegment, Awaitable[MessageSegment]]],
            ],
        ]
    ]

    data: dict
    _custom_builders: Dict[SupportedAdapters, CustomBuildFunc]

    def _register_custom_builder(
        self,
        adapter: SupportedAdapters,
        ms: Union[MessageSegment, CustomBuildFunc],
    ):
        if isinstance(ms, MessageSegment):
            self._custom_builders[adapter] = lambda _: ms
        else:
            self._custom_builders[adapter] = ms

    def _get_custom_builder(
        self,
        adapter: SupportedAdapters,
    ) -> Optional[CustomBuildFunc]:
        return self._custom_builders.get(adapter)

    def __init__(self) -> None:
        self._custom_builders = {}

    def __init_subclass__(cls) -> None:
        cls._builders = {}
        return super().__init_subclass__()

    def __eq__(self, other: Self) -> bool:
        return self.data == other.data

    def overwrite(
        self,
        adapter: SupportedAdapters,
        ms: Union[MessageSegment, CustomBuildFunc],
    ) -> Self:
        """为某个 adapter 重写产生的 MessageSegment 或重写产生 MessageSegment 的方法"""
        self._register_custom_builder(adapter, ms)
        return self

    async def build(self, bot: Bot) -> MessageSegment:
        adapter_name = extract_adapter_type(bot)
        if custom_builder := self._get_custom_builder(adapter_name):
            return await do_build_custom(custom_builder, bot)
        if builder := self._builders[adapter_name]:
            return await do_build(self, builder, bot)
        raise AdapterNotInstalled(adapter_name)

    def __add__(self: TMSF, other: Union[str, TMSF, Iterable[TMSF]]):
        return MessageFactory(self) + other

    def __radd__(self: TMSF, other: Union[str, TMSF, Iterable[TMSF]]):
        return MessageFactory(other) + self

    async def send(self, *, at_sender=False, reply=False):
        """回复消息，仅能用在事件响应器中"""
        return await MessageFactory(self).send(at_sender=at_sender, reply=reply)

    async def send_to(self, target: PlatformTarget, bot: Optional[Bot] = None):
        """主动发送消息，将消息发送到 target，如果不传入 bot 将自动选择 bot（此功能需要显式开启）"""
        return await MessageFactory(self).send_to(target, bot)

    async def finish(self, *, at_sender=False, reply=False, **kwargs) -> NoReturn:
        """与 `matcher.finish()` 作用相同，仅能用在事件响应器中"""
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        raise FinishedException

    async def pause(self, *, at_sender=False, reply=False, **kwargs) -> NoReturn:
        """与 `matcher.pause()` 作用相同，仅能用在事件响应器中"""
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        raise PausedException

    async def reject(self, *, at_sender=False, reply=False, **kwargs) -> NoReturn:
        """与 `matcher.reject()` 作用相同，仅能用在事件响应器中"""
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        raise RejectedException

    async def reject_arg(
        self, key: str, *, at_sender=False, reply=False, **kwargs
    ) -> NoReturn:
        """与 `matcher.reject_arg()` 作用相同，仅能用在事件响应器中"""
        matcher = current_matcher.get()
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        await matcher.reject_arg(key)

    async def reject_receive(
        self, key: str, *, at_sender=False, reply=False, **kwargs
    ) -> NoReturn:
        """与 `matcher.reject_receive()` 作用相同，仅能用在事件响应器中"""
        matcher = current_matcher.get()
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        await matcher.reject_receive(key)


class MessageFactory(List[TMSF]):
    _text_factory: Callable[[str], TMSF]
    _message_registry: Dict[SupportedAdapters, Type[Message]] = {}

    @classmethod
    def register_text_ms(cls, factory: Callable[[str], TMSF]):
        cls._text_factory = factory
        return factory

    @classmethod
    def get_text_factory(cls):
        return cls._text_factory

    @classmethod
    def register_adapter_message(
        cls,
        adapter: SupportedAdapters,
        message_class: Type[Message],
    ):
        cls._message_registry[adapter] = message_class

    async def build(self, bot: Bot) -> Message:
        warn(DeprecationWarning("MessageFactory.build is deprecated"))
        return await self._build(bot)

    async def _build(self, bot: Bot) -> Message:
        adapter_name = extract_adapter_type(bot)
        if message_type := self._message_registry.get(adapter_name):
            ms: List[MessageSegment] = await asyncio.gather(
                *[ms_factory.build(bot) for ms_factory in self],
            )
            return message_type(ms)
        raise AdapterNotInstalled(adapter_name)

    def __init__(self, message: Union[str, Iterable[TMSF], TMSF]):
        super().__init__()

        if message is None:
            return

        if isinstance(message, str):
            self.append(self.get_text_factory()(message))
        elif isinstance(message, MessageSegmentFactory):
            self.append(message)
        elif isinstance(message, Iterable):
            self.extend(message)

    def __add__(self: TMF, other: Union[str, TMSF, Iterable[TMSF]]) -> TMF:
        result = self.copy()
        result += other
        return result

    def __radd__(self: TMF, other: Union[str, TMSF, Iterable[TMSF]]) -> TMF:
        result = self.__class__(other)
        return result + self

    def __iadd__(self: TMF, other: Union[str, TMSF, Iterable[TMSF]]) -> TMF:
        if isinstance(other, str):
            self.append(self.get_text_factory()(other))
        elif isinstance(other, MessageSegmentFactory):
            self.append(other)
        elif isinstance(other, Iterable):
            self.extend(other)

        return self

    def append(self: TMF, obj: Union[str, TMSF]) -> TMF:
        if isinstance(obj, MessageSegmentFactory):
            super().append(obj)
        elif isinstance(obj, str):
            super().append(self.get_text_factory()(obj))

        return self

    def extend(self: TMF, obj: Union[TMF, Iterable[TMSF]]) -> TMF:
        for message_segment_factory in obj:
            self.append(message_segment_factory)

        return self

    def copy(self: TMF) -> TMF:
        return deepcopy(self)

    async def send(self, *, at_sender=False, reply=False) -> "Receipt":
        """回复消息，仅能用在事件响应器中"""

        try:
            event = current_event.get()
            bot = current_bot.get()
        except LookupError as e:
            raise RuntimeError("send() 仅能在事件响应器中使用，主动发送消息请使用 send_to") from e

        target = extract_target(event)
        return await self._do_send(bot, target, event, at_sender, reply)

    async def send_to(
        self, target: PlatformTarget, bot: Optional[Bot] = None
    ) -> "Receipt":
        """主动发送消息，将消息发送到 target，如果不传入 bot 将自动选择 bot（此功能需要显式开启）"""
        
        if bot is None:
            bot = get_bot(target)
        return await self._do_send(bot, target, None, False, False)

    async def finish(self, *, at_sender=False, reply=False, **kwargs) -> NoReturn:
        """与 `matcher.finish()` 作用相同，仅能用在事件响应器中"""
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        raise FinishedException

    async def pause(self, *, at_sender=False, reply=False, **kwargs) -> NoReturn:
        """与 `matcher.pause()` 作用相同，仅能用在事件响应器中"""
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        raise PausedException

    async def reject(self, *, at_sender=False, reply=False, **kwargs) -> NoReturn:
        """与 `matcher.reject()` 作用相同，仅能用在事件响应器中"""
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        raise RejectedException

    async def reject_arg(
        self, key: str, *, at_sender=False, reply=False, **kwargs
    ) -> NoReturn:
        """与 `matcher.reject_arg()` 作用相同，仅能用在事件响应器中"""
        matcher = current_matcher.get()
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        await matcher.reject_arg(key)

    async def reject_receive(
        self, key: str, *, at_sender=False, reply=False, **kwargs
    ) -> NoReturn:
        """与 `matcher.reject_receive()` 作用相同，仅能用在事件响应器中"""
        matcher = current_matcher.get()
        await self.send(at_sender=at_sender, reply=reply, **kwargs)
        await matcher.reject_receive(key)

    async def _do_send(
        self,
        bot: Bot,
        target: PlatformTarget,
        event: Optional[Event],
        at_sender: bool,
        reply: bool,
    ) -> "Receipt":
        adapter = extract_adapter_type(bot)
        if not (sender := sender_map.get(adapter)):
            raise RuntimeError(
                f"send method for {adapter} not registered",
            )  # pragma: no cover
        return await sender(bot, self, target, event, at_sender, reply)


AggregatedSender = Callable[
    [Bot, List[MessageFactory], PlatformTarget, Optional[Event]],
    Awaitable[None],
]


class AggregatedMessageFactory:
    message_factories: List[MessageFactory]
    sender: ClassVar[Dict[SupportedAdapters, AggregatedSender]] = {}

    def __init__(
        self,
        msgs: List[Union[MessageFactory, MessageSegmentFactory]],
    ) -> None:
        self.message_factories = []
        for msg in msgs:
            if isinstance(msg, MessageSegmentFactory):
                self.message_factories.append(MessageFactory(msg))
            else:
                self.message_factories.append(msg)

    def __eq__(self, other: Self):
        return self.message_factories == other.message_factories

    @classmethod
    def register_aggregated_sender(cls, adapter: SupportedAdapters):
        def wrapper(func: AggregatedSender):
            cls.sender[adapter] = func
            return func

        return wrapper

    async def _send_aggregated_message_default(
        self,
        bot: Bot,
        target: PlatformTarget,
        event: Optional[Event],
    ):
        for msg_fac in self.message_factories:
            await msg_fac._do_send(bot, target, event, False, False)  # noqa: SLF001

    async def _do_send(self, bot: Bot, target: PlatformTarget, event: Optional[Event]):
        adapter = extract_adapter_type(bot)
        if sender := self.__class__.sender.get(adapter):  # custom aggregate sender
            try:
                return await sender(bot, self.message_factories, target, event)
            except FallbackToDefault:
                await self._send_aggregated_message_default(bot, target, event)
        # fallback
        await self._send_aggregated_message_default(bot, target, event)
        return None

    async def send(self):
        """回复消息，仅能用在事件响应器中"""
        try:
            event = current_event.get()
            bot = current_bot.get()
        except LookupError as e:
            raise RuntimeError("send() 仅能在事件响应器中使用，主动发送消息请使用 send_to") from e

        target = extract_target(event)
        await self._do_send(bot, target, event)

    async def send_to(self, target: PlatformTarget, bot: Optional[Bot] = None):
        """主动发送消息，将消息发送到 target，如果不传入 bot 将自动选择 bot（此功能需要显式开启）"""
        if bot is None:
            bot = get_bot(target)
        await self._do_send(bot, target, None)

    async def finish(self, **kwargs) -> NoReturn:
        """与 `matcher.finish()` 作用相同，仅能用在事件响应器中"""
        await self.send(**kwargs)
        raise FinishedException

    async def pause(self, **kwargs) -> NoReturn:
        """与 `matcher.pause()` 作用相同，仅能用在事件响应器中"""
        await self.send(**kwargs)
        raise PausedException

    async def reject(self, **kwargs) -> NoReturn:
        """与 `matcher.reject()` 作用相同，仅能用在事件响应器中"""
        await self.send(**kwargs)
        raise RejectedException

    async def reject_arg(self, key: str, **kwargs) -> NoReturn:
        """与 `matcher.reject_arg()` 作用相同，仅能用在事件响应器中"""
        matcher = current_matcher.get()
        await self.send(**kwargs)
        await matcher.reject_arg(key)

    async def reject_receive(self, key: str, **kwargs) -> NoReturn:
        """与 `matcher.reject_receive()` 作用相同，仅能用在事件响应器中"""
        matcher = current_matcher.get()
        await self.send(**kwargs)
        await matcher.reject_receive(key)


def register_ms_adapter(
    adapter: SupportedAdapters,
    ms_factory: Type[TMSF],
) -> Callable[[BuildFunc], BuildFunc]:
    def decorator(builder: BuildFunc) -> BuildFunc:
        ms_factory._builders[adapter] = builder  # noqa: SLF001
        return builder

    return decorator


def assamble_message_factory(
    origin_msg_factory: MessageFactory,
    mention_message_segment: Optional[MessageSegmentFactory],
    reply_message_segment: Optional[MessageSegmentFactory],
    at_sender: bool,
    reply: bool,
) -> MessageFactory:
    full_message_factory = MessageFactory([])
    if reply_message_segment and reply:
        full_message_factory += reply_message_segment
    if mention_message_segment and at_sender:
        full_message_factory += mention_message_segment
    full_message_factory += origin_msg_factory

    return full_message_factory



from __future__ import annotations

from typing import Annotated, Tuple, Dict, Optional, Any  # noqa: F401

from nonebot import Bot
from nonebot import get_driver
from nonebot.internal.params import Depends
from nonebot.params import Event
from nonebot_plugin_alconna import UniMsg, Reply
from nonebot_plugin_bind.utils import get_user
from nonebot.exception import SkippedException
from ..type_store.msgs_link import msg_links
from ..type_store.user_chat import users, User, public_user




async def extract_image(event, bot):
    messages = event.get_message()
    adapter_name = bot.adapter.get_name()

    image = ""
    if adapter_name == 'OneBot V11':
        for message in messages:
            if message.type == 'image':
                image = message.data.get("url")
                break
    elif adapter_name == 'Telegram':
        for message in messages:
            if message.type == "photo":
                image_data = await bot.call_api("get_file", file_id=message.data.get("file"))
                image = f"https://api.telegram.org/file/bot{bot.bot_config.token}/{image_data.file_path}"
                break

    elif adapter_name == "Discord":
        if event.attachments:
            for attachment in event.attachments:
                if attachment.content_type.startswith("image"):
                    image = attachment.proxy_url or attachment.url
                    break
    elif adapter_name == 'QQ Guild':
        for message in messages:
            if message.type == 'attachment':
                image = message.data.get('url')
                break
    return image

def GetUser():
    async def dependency(bot: Bot, event: Event):
        bind_user = await get_user(bot, event, auto_create=True)
        return users[bind_user.id]

    return Depends(dependency)


A_User = Annotated[User, GetUser()]

def GetUser():
    async def dependency(bot: Bot, event: Event):
        bind_user = await get_user(bot, event, auto_create=True)
        return users[bind_user.id]

    return Depends(dependency)


A_User = Annotated[User, GetUser()]


def GetQuestionChatUser():
    async def dependency(bot: Bot, event: Event, unimsg: UniMsg):
        from ..type_store.web_config import common_config

        msg = event.get_plaintext()
        common_config = common_config()
        if msg.startswith(common_config.private_command):
            bind_user = await get_user(bot, event, auto_create=True)
            user = users[bind_user.id]
            prefix = common_config.private_command
        elif msg.startswith(common_config.public_command):
            user = public_user
            prefix = common_config.public_command
        else:
            try:
                bind_user = await get_user(bot, event, auto_create=True)
                user = users[bind_user.id]
                prefix = common_config.private_command
                message_id = unimsg[Reply][0].id
                chat = msg_links.get_chatbot(user.user_id, message_id)
                return event.get_plaintext(), chat, user
            except Exception:
                try:
                    message_id = unimsg[Reply][0].id
                    chat = msg_links.get_chatbot(public_user.user_id, message_id)
                    return event.get_plaintext(), chat, public_user
                except Exception:
                    pass
        for _, chat in user.bots.items():
            full_prefix = f"{prefix}{chat.name}"
            if msg.startswith(full_prefix):
                return msg[len(full_prefix):], chat, user
        raise SkippedException
    return Depends(dependency)


QCU = Annotated[Tuple[str, Any, User], GetQuestionChatUser()]



from typing import Optional

from nonebot_plugin_templates.templates_render import dict_render

Classes = []


def register_chatbot(cls):
    global Classes
    Classes.append(cls)
    return cls


from . import  PoeBot,ChatGPTApiBot,ChatGPTWebBot,BingBot,ClaudeAiBot,SlackClaudeBot  # noqa: E402, F401


class ChatBots:
    image_: bytes = None
    dict_hash: Optional[int] = None

    def get_model_class(self, model: str):
        able_models = [key for key, value in self.chatbot_dict.items() if value["able"]]
        if model.isdigit():
            if len(able_models) >= int(model):
                model = able_models[int(model) - 1]
            else:
                raise Exception("模型序号超出总数,请检查后重新输入")
        if model in able_models:
            return self.chatbot_dict[model]["chatbot"]
        else:
            raise Exception(
                f"模型不存在或不可用.\n当前可用模型有: {', '.join(model for model in able_models)}"
            )

    @property
    def chatbot_dict(self):
        c_d = {
            class_.__name__.replace("Bot", ""): {
                "able": class_.able,
                "description": class_.desc,
                "chatbot": class_,
            }
            for class_ in Classes
        }

        return c_d

    async def get_image(self):
        dict_hash = hash(str(self.chatbot_dict))
        if dict_hash == self.dict_hash and self.image_:
            return self.image_
        else:
            data = {
                k: v["description"] for k, v in self.chatbot_dict.items() if v["able"]
            }

            self.image_ = await dict_render(data, title="模型列表")
            self.dict_hash = dict_hash
            return self.image_


chatbots = ChatBots()



import asyncio
from functools import wraps

from nonebot_plugin_web_config import get_config


def config_func(namespace):
    def decorator(cls):
        func_name = cls.__name__.lower() + "_config"

        @wraps(cls)
        def func():
            return get_config(module=__name__, config_name=cls.__config_name__)

        namespace[func_name] = func
        return cls

    return decorator


def retry(tries, error_message):
    def decorator(func):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            error = None
            for _ in range(tries):
                try:
                    if asyncio.iscoroutinefunction(func):
                        return await func(*args, **kwargs)
                    else:
                        return func(*args, **kwargs)
                except Exception as e:
                    error = e
            raise Exception(f"{error_message}: {error}")

        return wrapper

    return decorator



from .decorators import retry, config_func  # noqa: F401
from .generate import generate_random  # noqa: F401
from .pastebin import get_url  # noqa: F401
from .render import md_to_pic  # noqa: F401
from .valid import is_valid_name  # noqa: F401
from nonebot import get_driver

Command_Start = list(get_driver().config.command_start)[0]


from functools import partial
from io import BytesIO
from pathlib import Path
from typing import cast, Literal, Union, Optional, List, Sequence, Any

import anyio
from nonebot import logger
from nonebot.adapters import Bot, Event
from nonebot.adapters.telegram.model import MessageEntity, InputMedia


from ..types import Text, Image, Reply, Mention
from ..utils import (
    MessageFactory,
    SupportedAdapters,
    TargetTelegramForum,
    TargetTelegramCommon,
    MessageSegmentFactory,
    register_sender,
    register_get_bot_id,
    register_ms_adapter,
    assamble_message_factory,
    register_target_extractor, Receipt, get_bot_id,
)

try:
    from nonebot.adapters.telegram import Bot as BotTG
    from nonebot.adapters.telegram.message import File, Entity, UnCombinFile
    from nonebot.adapters.telegram import Message, MessageSegment
    from nonebot.adapters.telegram.event import (
        MessageEvent,
        ChannelPostEvent,
        GroupMessageEvent,
        PrivateMessageEvent,
        ForumTopicMessageEvent,
    )

    adapter = SupportedAdapters.telegram
    register_telegram = partial(register_ms_adapter, adapter)

    MessageFactory.register_adapter_message(SupportedAdapters.telegram, Message)


    def build_entities_form_msg(
        message: Sequence[MessageSegment]
    ) -> Optional[List[MessageEntity]]:
        return (
            (
                [
                    MessageEntity(
                        type=entity.type,
                        offset=sum(map(len, message[:i])),
                        length=len(entity.data["text"]),
                        url=entity.data.get("url"),
                        user=entity.data.get("user"),
                        language=entity.data.get("language"),
                    )
                    for i, entity in enumerate(message)
                    if entity.is_text() and entity.type != "text"
                ]
                or None
            )
            if message
            else None
        )


    @register_telegram(Text)
    def _text(t: Text) -> MessageSegment:
        return Entity.text(t.data["text"])


    @register_telegram(Image)
    async def _image(i: Image) -> MessageSegment:
        image = i.data["image"]
        if isinstance(image, Path):
            image = await anyio.Path(image).read_bytes()
        if isinstance(image, BytesIO):
            image = image.getvalue()
        return File.photo(image)


    @register_telegram(Mention)
    async def _mention(m: Mention) -> MessageSegment:
        user_id = m.data["user_id"]
        return (
            Entity.mention(f"{user_id} ")
            if user_id.startswith("@")
            else Entity.text_link("用户 ", f"tg://user?id={user_id}")
        )


    @register_telegram(Reply)
    async def _reply(r: Reply) -> MessageSegment:
        return MessageSegment("reply", cast(dict, r.data))


    @register_target_extractor(PrivateMessageEvent)
    @register_target_extractor(GroupMessageEvent)
    @register_target_extractor(ChannelPostEvent)
    def _extract_private_msg_event(event: Event) -> TargetTelegramCommon:
        assert isinstance(event, MessageEvent)
        return TargetTelegramCommon(chat_id=event.chat.id)


    @register_target_extractor(ForumTopicMessageEvent)
    def _extract_forum_msg_event(event: Event) -> TargetTelegramForum:
        assert isinstance(event, ForumTopicMessageEvent)
        return TargetTelegramForum(
            chat_id=event.chat.id,
            message_thread_id=event.message_thread_id,
        )


    def build_mention_from_event(event: MessageEvent) -> MessageSegment:
        # has user
        if isinstance(
            event,
            (PrivateMessageEvent, GroupMessageEvent, ForumTopicMessageEvent),
        ):
            user = event.from_

            # has username
            if username := user.username:
                return Entity.mention(f"@{username} ")

            # no username
            last_name = f" {user.last_name}" if user.last_name else ""
            return Entity.text_link(
                f"{user.first_name}{last_name} ",
                f"tg://user?id={user.id}",
            )

        # no user
        return Entity.text("")


    class TelegramReceipt(Receipt):
        sent_msg: Any
        sent_type: Literal["text", "image"]
        chat_id: Union[str, int]
        reply_to_msg_id: Optional[Union[str, int]]
        mention_user_id: Optional[Union[str, int]]
        adapter_name = adapter

        @property
        def message_id(self) -> Union[str, list[str]]:
            if isinstance(self.sent_msg, list):
                return [f'{msg.message_id}.{self.chat_id}' for msg in self.sent_msg]
            else:
                return f'{self.sent_msg.message_id}.{self.chat_id}'

        async def revoke(self):
            bot = self._get_bot()

            async def del_msg_with_retry(msg_id):
                retry = 3
                error = Exception("Unknown Error Deleting Telegram message")
                while retry > 0:
                    try:
                        return await bot.delete_message(
                            chat_id=self.chat_id, message_id=msg_id
                        )
                    except Exception as e:
                        error = e
                        pass
                raise error

            if isinstance(self.sent_msg, list):
                for message_id in self.message_id:
                    await del_msg_with_retry(message_id.split(".")[0])
            else:
                await del_msg_with_retry(self.message_id.split(".")[0])

        async def edit(self, msg: MessageFactory[MessageSegmentFactory], at_sender=False, reply=False):  # noqa: E501
            bot = self._get_bot()
            if isinstance(msg, MessageSegmentFactory):
                msg = MessageFactory([msg])
            message_to_send = Message()
            for message_segment_factory in msg:
                if isinstance(message_segment_factory, Reply):
                    self.reply_to_msg_id = message_segment_factory.data["message_id"]
                else:
                    message_segment = await message_segment_factory.build(bot)
                    message_to_send += message_segment
            if self.reply_to_msg_id and reply:
                message_to_send = await Reply(self.reply_to_msg_id).build(bot) + message_to_send
            if self.mention_user_id and at_sender:
                message_to_send = await MessageFactory(self.mention_user_id).build(bot) + message_to_send  # noqa: E501

            # 处理 Message 的编辑
            # 分离各类型 seg
            entities = Message(x for x in message_to_send if isinstance(x, Entity))
            images = Message(
                x
                for x in message_to_send
                if isinstance(x, File) and not isinstance(message_to_send, UnCombinFile)
            )

            if self.sent_type == "image":
                if not images:
                    raise Exception("Saa在编辑Tg的含Image信息时,仅能编辑为含少于等于原Image数量的信息")
                if isinstance(self.sent_msg, list):
                    if len(images) > len(self.sent_msg):
                        logger.error("Saa在编辑Tg的含Image信息时,仅能编辑为含少于等于原Image数量的信息")
                    message_ids = self.message_id
                    for index, message_id in enumerate(message_ids):
                        if index < len(images):
                            await bot.edit_message_media(
                                media=InputMedia(type=images[index].type, media=images[index].data["file"],
                                                 caption=str(entities),
                                                 caption_entities=build_entities_form_msg(message_to_send)),
                                chat_id=self.chat_id,
                                message_id=message_ids[index].split(".")[0],
                            )
                        else:
                            to_del_receipt = self.copy()
                            to_del_receipt.sent_msg = self.sent_msg[index:]
                            await to_del_receipt.revoke()
                            self.sent_msg = self.sent_msg[:index]
                            break
                else:
                    if len(images) > 1:
                        logger.error("Saa在编辑消息时对于单图片的Tg消息,也只能编辑为单图片的消息")
                    await bot.edit_message_media(
                        media=InputMedia(type=images[0].type, media=images[0].data["file"],
                                         caption=str(entities),
                                         caption_entities=build_entities_form_msg(message_to_send)),
                        chat_id=self.chat_id,
                        message_id=self.message_id.split(".")[0],
                    )
            else:
                if images:
                    raise Exception("Saa在编辑Tg的纯Text信息时,仅能编辑为纯Text信息")
                return await bot.edit_message_text(
                    chat_id=self.chat_id,
                    message_id=self.message_id.split(".")[0],
                    text=str(entities),
                    entities=build_entities_form_msg(message_to_send),
                )


    @register_sender(SupportedAdapters.telegram)
    async def send(
        bot,
        msg: MessageFactory[MessageSegmentFactory],
        target,
        event,
        at_sender: bool,
        reply: bool,
    ):
        assert isinstance(bot, BotTG)
        assert isinstance(target, (TargetTelegramCommon, TargetTelegramForum))

        if event:
            assert isinstance(event, MessageEvent)
            full_msg = assamble_message_factory(
                msg,
                (
                    None
                    if isinstance(event, ChannelPostEvent)
                    else Mention(event.get_user_id())
                ),
                Reply(event.message_id),
                at_sender,
                reply,
            )
        else:
            full_msg = msg
        params = {"sent_type": "text", "mention_user_id": event.get_user_id(), "reply_to_msg_id": event.message_id}

        message_to_send = Message()
        reply_to_message_id = None
        for message_segment_factory in full_msg:
            if isinstance(message_segment_factory, Image):
                params["sent_type"] = "image"
            if isinstance(message_segment_factory, Reply):
                reply_to_message_id = int(message_segment_factory.data["message_id"])
                continue

            if (
                event
                and isinstance(message_segment_factory, Mention)
                and message_segment_factory.data["user_id"] == event.get_user_id()
            ):
                message_segment = build_mention_from_event(event)
            else:
                message_segment = await message_segment_factory.build(bot)

            message_to_send += message_segment

        chat_id = target.chat_id
        message_thread_id = (
            target.message_thread_id
            if isinstance(target, TargetTelegramForum)
            else None
        )
        sent_msg = await bot.send_to(
            chat_id,
            message_to_send,
            message_thread_id=message_thread_id,
            reply_to_message_id=reply_to_message_id,
        )
        return TelegramReceipt(bot_id=get_bot_id(bot), sent_msg=sent_msg, message_id=sent_msg.message_id,
                               chat_id=chat_id, **params)


    @register_get_bot_id(adapter)
    def _get_id(bot: Bot):
        assert isinstance(bot, BotTG)
        return bot.self_id

    @register_get_bot_id(adapter)
    def _get_id(bot: Bot):
        assert isinstance(bot, BotTG)
        return bot.self_id

except ImportError:
    pass
except Exception:
    raise



from strenum import StrEnum


class SupportedAdapters(StrEnum):
    onebot_v11 = "OneBot V11"
    onebot_v12 = "OneBot V12"
    qqguild = "QQ Guild"
    kaiheila = "Kaiheila"
    telegram = "Telegram"
    feishu = "Feishu"
    discord = "Discord"

    fake = "fake"  # for nonebug


class SupportedPlatform(StrEnum):
    qq_group = "QQ Group"
    qq_private = "QQ Private"
    qq_guild_channel = "QQ Guild Channel"
    qq_guild_direct = "QQ Guild Direct"
    kaiheila_channel = "Kaiheila Channel"
    kaiheila_private = "Kaiheila Private"
    unknown_ob12 = "Unknow Onebot 12 Platform"
    telegram_common = "Telegram Common"
    telegram_forum = "Telegram Forum"
    feishu_private = "Feishu Private"
    feishu_group = "Feishu Group"
    discord_channel = "Discord Channel"


supported_adapter_names = set(SupportedAdapters._member_map_.values())  # noqa: SLF001



""" 提供获取 Bot 的方法 """
import random
from collections import defaultdict
from typing import Dict, List, Callable, Optional, Awaitable

from nonebot import get_bots
from nonebot import get_driver
from nonebot.adapters import Bot

from .const import SupportedAdapters
from .exceptions import NoBotFound
from .helpers import extract_adapter_type
from .platform_send_target import PlatformTarget, TargetQQGuildDirect

BOT_CACHE: Dict[PlatformTarget, List[Bot]] = defaultdict(list)
BOT_CACHE_DICT: Dict[SupportedAdapters, Dict[str, Bot]] = defaultdict(dict)

ListTargetsFunc = Callable[[Bot], Awaitable[List[PlatformTarget]]]

list_targets_map: Dict[str, ListTargetsFunc] = {}

inited = False

GetBotIdFunc = Callable[[Bot], str]
_bot_id_getter: Dict[SupportedAdapters, GetBotIdFunc] = {}


def _assert_inited():
    if not inited:
        raise RuntimeError("自动选择 Bot 的功能未启用，请先调用 enable_auto_select_bot 启用此功能")


def register_get_bot_id(adapter_name: SupportedAdapters):
    """注册得到 Bot 唯一标示符的方式"""

    def wrapper(func: GetBotIdFunc):
        _bot_id_getter[adapter_name] = func
        return func

    return wrapper


def get_bot_id(bot: Bot) -> str:
    getter = _bot_id_getter.get(extract_adapter_type(bot))
    if not getter:
        raise RuntimeError(f"get bot id of {type(bot)} is not supported")

    return getter(bot)


def get_bot_by_id(adapter_name: SupportedAdapters, id: str) -> Optional[Bot]:
    """通过唯一标示符选择 Bot，需要开启 auto_select_bot"""
    if adapter_name not in _bot_id_getter.keys():
        raise RuntimeError(f"get bot id of {adapter_name} is not supported")
    return BOT_CACHE_DICT[adapter_name].get(id)


driver = get_driver()


@driver.on_bot_connect
@driver.on_bot_disconnect
async def __(bot: Bot):
    await init_bots()


def _register_hook():
    from nonebot import get_driver

    driver = get_driver()

    @driver.on_bot_connect
    @driver.on_bot_disconnect
    async def _(bot: Bot):
        await refresh_bots()


def enable_auto_select_bot():
    """启用自动选择 Bot 的功能

    启用后，发送主动消息时，可不提供 Target，会自动选择一个 Bot 进行发送
    """
    global inited

    if inited:
        return

    _register_hook()
    inited = True


def register_list_targets(adapter: SupportedAdapters):
    def wrapper(func: ListTargetsFunc):
        list_targets_map[adapter] = func
        return func

    return wrapper


async def refresh_bots():
    """刷新缓存的 Bot 数据"""
    BOT_CACHE.clear()
    BOT_CACHE_DICT.clear()
    for bot in list(get_bots().values()):
        adapter_name = extract_adapter_type(bot)

        if list_targets := list_targets_map.get(adapter_name):
            targets = await list_targets(bot)
            for target in targets:
                BOT_CACHE[target].append(bot)


async def init_bots():
    """初始化bot_id和bot的对应关系"""
    BOT_CACHE.clear()
    BOT_CACHE_DICT.clear()
    for bot in list(get_bots().values()):
        adapter_name = extract_adapter_type(bot)

        if adapter_name in _bot_id_getter.keys():
            BOT_CACHE_DICT[adapter_name][get_bot_id(bot)] = bot


def get_bot(target: PlatformTarget) -> Bot:
    """获取 Bot"""
    _assert_inited()

    # TODO: 通过更方便的方式判断当前 Target 是否支持
    if isinstance(target, TargetQQGuildDirect):
        raise NotImplementedError("暂不支持私聊")

    bots = BOT_CACHE.get(target)
    if not bots:
        raise NoBotFound()

    return random.choice(bots)
